import {NgModule} from "@angular/core";
import {RouterModule} from "@angular/router";
import {PageNotFoundPresentational} from "@app/components/presentationals/page-not-found/page-not-found.presentational";
import {ApplicationRolePermissionEnum} from "@app/shared/enums/application-role-permission.enum";
import {AppRoutesEnum} from "@app/shared/enums/routes.enum";
import {ApplicationRoleGuardService} from "@app/shared/guards/application-role-guard.service";
import {AuthGuardService} from "@app/shared/guards/auth-guard.service";
import {DlcmRoutes} from "@app/shared/models/dlcm-route.model";
import {TRANSLATE} from "solidify-frontend";

const routes: DlcmRoutes = [
  {
    path: AppRoutesEnum.homePage,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./features/home/home.module").then(m => m.HomeModule),
    data: {
      permission: ApplicationRolePermissionEnum.noPermission,
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: AppRoutesEnum.deposit,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./features/deposit/deposit.module").then(m => m.DepositModule),
    data: {
      breadcrumb: TRANSLATE("breadcrumb.deposit.root"),
      permission: ApplicationRolePermissionEnum.userPermission,
    },
    canActivate: [AuthGuardService, ApplicationRoleGuardService],
  },
  {
    path: AppRoutesEnum.organizationalUnit,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./features/organizational-unit/organizational-unit.module").then(m => m.OrganizationalUnitModule),
    data: {
      breadcrumb: TRANSLATE("breadcrumb.organizationalUnit.root"),
      permission: ApplicationRolePermissionEnum.adminPermission,
    },
    canActivate: [AuthGuardService],
  },
  {
    path: AppRoutesEnum.admin,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./features/admin/admin.module").then(m => m.AdminModule),
    data: {
      breadcrumb: TRANSLATE("breadcrumb.admin.root"),
    },
    canActivate: [AuthGuardService, ApplicationRoleGuardService],
  },
  {
    path: AppRoutesEnum.root,
    redirectTo: AppRoutesEnum.homePage,
    pathMatch: "full",
  },
  {
    path: AppRoutesEnum.index,
    redirectTo: AppRoutesEnum.homePage,
    pathMatch: "full",
  },
  {
    path: AppRoutesEnum.homePage,
    redirectTo: AppRoutesEnum.homePage,
    pathMatch: "full",
  },
  {
    path: "**",
    component: PageNotFoundPresentational,
  },
];

@NgModule({
  imports: [RouterModule.forRoot(routes, {useHash: true})],
  exports: [RouterModule],
})
export class AppRoutingModule {
}
