import {adminInstitutionActionNameSpace} from "@admin/institution/stores/admin-institution.action";
import {Institution} from "@app/generated-api";
import {
  Actions,
  Selector,
  State,
  Store,
} from "@ngxs/store";
import {AdminResourceApiEnum} from "@shared/enums/api.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {
  ApiService,
  defaultResourceStateInitValue,
  isNullOrUndefined,
  NotificationService,
  ResourceState,
  ResourceStateModel,
  StoreUtil,
  TRANSLATE,
} from "solidify-frontend";

export interface AdminInstitutionStateModel extends ResourceStateModel<Institution> {

}

@State<AdminInstitutionStateModel>({
  name: LocalStateEnum.admin_institution,
  defaults: {
    ...defaultResourceStateInitValue,
  },
})
export class AdminInstitutionState extends ResourceState<Institution> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: adminInstitutionActionNameSpace,
      routeRedirectUrlAfterSuccessAction: RoutesEnum.adminInstitution,
      notificationResourceCreateTextToTranslate: TRANSLATE("admin.institution.notification.resource.create"),
      notificationResourceDeleteTextToTranslate: TRANSLATE("admin.institution.notification.resource.delete"),
      notificationResourceUpdateTextToTranslate: TRANSLATE("admin.institution.notification.resource.update"),
    });
  }

  protected get _urlResource(): string {
    return AdminResourceApiEnum.institutions;
  }

  @Selector()
  static isLoading(state: AdminInstitutionStateModel): boolean {
    return StoreUtil.isLoadingState(state);
  }

  @Selector()
  static isLoadingWithDependency(state: AdminInstitutionStateModel): boolean {
    return this.isLoading(state);
  }

  @Selector()
  static isReadyToBeDisplayed(state: AdminInstitutionStateModel): boolean {
    return this.isReadyToBeDisplayedInCreateMode
      && !isNullOrUndefined(state.current);
  }

  @Selector()
  static isReadyToBeDisplayedInCreateMode(state: AdminInstitutionStateModel): boolean {
    return true;
  }
}
