import {adminOAuth2ClientActionNameSpace} from "@admin/oauth2-client/stores/admin-oauth2-client.action";
import {Oauth2Client} from "@app/generated-api";
import {
  Actions,
  Selector,
  State,
  Store,
} from "@ngxs/store";
import {AdminResourceApiEnum} from "@shared/enums/api.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {
  ApiService,
  defaultResourceStateInitValue,
  isNullOrUndefined,
  NotificationService,
  ResourceState,
  ResourceStateModel,
  StoreUtil,
  TRANSLATE,
} from "solidify-frontend";

export interface AdminOAuth2ClientStateModel extends ResourceStateModel<Oauth2Client> {
}

@State<AdminOAuth2ClientStateModel>({
  name: LocalStateEnum.admin_oauth2Client,
  defaults: {
    ...defaultResourceStateInitValue,
  },
})
export class AdminOAuth2ClientState extends ResourceState<Oauth2Client> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: adminOAuth2ClientActionNameSpace,
      routeRedirectUrlAfterSuccessAction: RoutesEnum.admin,
      notificationResourceCreateTextToTranslate: TRANSLATE("admin.oauth2.notification.resource.create"),
      notificationResourceDeleteTextToTranslate: TRANSLATE("admin.oauth2.notification.resource.delete"),
      notificationResourceUpdateTextToTranslate: TRANSLATE("admin.oauth2.notification.resource.update"),
    });
  }

  protected get _urlResource(): string {
    return AdminResourceApiEnum.oauth2Clients;
  }

  @Selector()
  static isLoading(state: AdminOAuth2ClientStateModel): boolean {
    return StoreUtil.isLoadingState(state);
  }

  @Selector()
  static isLoadingWithDependency(state: AdminOAuth2ClientStateModel): boolean {
    return this.isLoading(state);
  }

  @Selector()
  static isReadyToBeDisplayed(state: AdminOAuth2ClientStateModel): boolean {
    return this.isReadyToBeDisplayedInCreateMode
      && !isNullOrUndefined(state.current);
  }

  @Selector()
  static isReadyToBeDisplayedInCreateMode(state: AdminOAuth2ClientStateModel): boolean {
    return true;
  }
}
