import {PersonExtended} from "@admin/models/person-extended.model";
import {PersonRole} from "@admin/models/person-role.model";
import {adminOrgUnitActionNameSpace} from "@admin/orgunit/stores/admin-organizational-unit.action";
import {
  AdminOrganizationalUnitState,
  AdminOrganizationalUnitStateModel,
} from "@admin/orgunit/stores/admin-organizational-unit.state";
import {
  AdminOrgUnitFundingAgencyAction,
} from "@admin/orgunit/stores/funding-agency/admin-organizational-unit-funding-agency.action";
import {AdminOrgUnitInstitutionAction} from "@admin/orgunit/stores/institution/admin-organizational-unit-institution.action";
import {AdminOrgUnitPersonRoleAction} from "@admin/orgunit/stores/person-role/admin-organizational-unit-person-role.action";
import {AdminOrgUnitPreservationPolicyAction} from "@admin/orgunit/stores/preservation-policy/admin-organizational-unit-preservation-policy.action";
import {AdminOrgUnitSubmissionPolicyAction} from "@admin/orgunit/stores/submission-policy/admin-organizational-unit-submission-policy.action";
import {
  ChangeDetectionStrategy,
  Component,
} from "@angular/core";
import {MatDialog} from "@angular/material";
import {ActivatedRoute} from "@angular/router";
import {
  FundingAgency,
  Institution,
  PreservationPolicy,
  Role,
  SubmissionPolicy,
} from "@app/generated-api";
import {OrganizationalUnitExtended} from "@deposit/models/organizational-unit-extended.model";
import {
  Select,
  Store,
} from "@ngxs/store";
import {SharedAbstractDetailRoutable} from "@shared/components/routables/shared-abstract-detail/shared-abstract-detail.routable";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {LocalStateModel} from "@shared/models/local-state.model";
import {Observable} from "rxjs";
import {TRANSLATE} from "solidify-frontend";

@Component({
  selector: "dlcm-admin-orgunit-detail-routable",
  templateUrl: "./admin-orgunit-detail.routable.html",
  styleUrls: ["../../../../../../shared/components/routables/shared-abstract-detail/shared-abstract-detail.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AdminOrgunitDetailRoutable extends SharedAbstractDetailRoutable<OrganizationalUnitExtended, AdminOrganizationalUnitStateModel> {
  @Select(AdminOrganizationalUnitState.isLoadingWithDependency) isLoadingWithDependencyObs: Observable<boolean>;
  @Select(AdminOrganizationalUnitState.isReadyToBeDisplayed) isReadyToBeDisplayedObs: Observable<boolean>;
  @Select((state: LocalStateModel) => state.admin.admin_organizationalUnit.admin_organizationalUnit_submissionPolicy.selected) selectedSubmissionPoliciesObs: Observable<SubmissionPolicy[]>;
  @Select((state: LocalStateModel) => state.admin.admin_organizationalUnit.admin_organizationalUnit_preservationPolicy.selected) selectedPreservationPoliciesObs: Observable<PreservationPolicy[]>;
  @Select((state: LocalStateModel) => state.admin.admin_organizationalUnit.admin_organizationalUnit_fundingAgency.selected) selectedFundingAgenciesObs: Observable<FundingAgency[]>;
  @Select((state: LocalStateModel) => state.shared.shared_submissionPolicy.list) listSubmissionPoliciesObs: Observable<SubmissionPolicy[]>;
  @Select((state: LocalStateModel) => state.shared.shared_preservationPolicy.list) listPreservationPoliciesObs: Observable<PreservationPolicy[]>;
  @Select((state: LocalStateModel) => state.shared.shared_person.list) listPersonObs: Observable<PersonExtended[]>;
  @Select((state: LocalStateModel) => state.shared.shared_role.list) listRoleObs: Observable<Role[]>;
  @Select((state: LocalStateModel) => state.shared.shared_fundingAgency.list) listFundingAgenciesObs: Observable<FundingAgency[]>;
  @Select((state: LocalStateModel) => state.shared.shared_institution.list) listInstitutionsObs: Observable<Institution[]>;
  @Select((state: LocalStateModel) => state.admin.admin_organizationalUnit.admin_organizationalUnit_personRole.selected) selectedPersonRoleObs: Observable<PersonRole[]>;
  @Select((state: LocalStateModel) => state.admin.admin_organizationalUnit.admin_organizationalUnit_institution.selected) selectedInstitutionsObs: Observable<Institution[]>;

  readonly KEY_PARAM_NAME: string = "name";
  readonly KEY_DELETE_BUTTON: string = TRANSLATE("admin.organizationalUnit.delete");
  readonly KEY_EDIT_BUTTON: string = TRANSLATE("admin.organizationalUnit.edit");

  constructor(protected store: Store,
              protected route: ActivatedRoute,
              public dialog: MatDialog) {
    super(store, route, dialog, LocalStateEnum.admin_organizationalUnit, adminOrgUnitActionNameSpace, LocalStateEnum.admin);
  }

  getSubResourceWithParentId(id: string): void {
    this.store.dispatch(new AdminOrgUnitSubmissionPolicyAction.GetAll(id));
    this.store.dispatch(new AdminOrgUnitPreservationPolicyAction.GetAll(id));
    this.store.dispatch(new AdminOrgUnitPersonRoleAction.GetAll(id));
    this.store.dispatch(new AdminOrgUnitFundingAgencyAction.GetAll(id));
    this.store.dispatch(new AdminOrgUnitInstitutionAction.GetAll(id));
  }
}
