import {OrganizationalUnitPersonRole} from "@admin/models/organizational-unit-person-role.model";
import {PersonRole} from "@admin/models/person-role.model";
import {adminOrgUnitPersonRoleActionNameSpace} from "@admin/orgunit/stores/person-role/admin-organizational-unit-person-role.action";
import {ApiResourceNameEnum} from "@app/shared/enums/api-resource-name.enum";
import {AdminResourceApiEnum} from "@app/shared/enums/api.enum";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";

import {
  Actions,
  State,
  Store,
} from "@ngxs/store";
import {
  ApiService,
  defaultRelation3TiersStateInitValue,
  NotificationService,
  Relation3TiersForm,
  Relation3TiersState,
  Relation3TiersStateModel,
} from "solidify-frontend";

export interface AdminOrganizationalUnitPersonRoleStateModel extends Relation3TiersStateModel<PersonRole> {
}

@State<AdminOrganizationalUnitPersonRoleStateModel>({
  name: LocalStateEnum.admin_organizationalUnit_personRole,
  defaults: {
    ...defaultRelation3TiersStateInitValue,
  },
})
// OrganizationalUnitPersonController
export class AdminOrganizationalUnitPersonRoleState extends Relation3TiersState<PersonRole, OrganizationalUnitPersonRole> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: adminOrgUnitPersonRoleActionNameSpace,
      resourceName: ApiResourceNameEnum.PERSON,
    });
  }

  protected get _urlResource(): string {
    return AdminResourceApiEnum.organizationalUnits;
  }

  protected convertResourceInForm(resource: PersonRole): Relation3TiersForm {
    return {id: resource.resId, listId: resource.roles.map(r => r.resId)};
  }
}
