import {OrganizationalUnitRole} from "@admin/models/organizational-unit-role.model";
import {PersonExtended} from "@admin/models/person-extended.model";
import {adminPersonActionNameSpace} from "@admin/person/stores/admin-person.action";
import {
  AdminPersonState,
  AdminPersonStateModel,
} from "@admin/person/stores/admin-person.state";
import {AdminPersonInstitutionsAction} from "@admin/person/stores/institutions/admin-people-institutions.action";
import {AdminPersonOrgUnitRoleAction} from "@admin/person/stores/person-role/admin-person-orgunit-role.action";
import {
  ChangeDetectionStrategy,
  Component,
} from "@angular/core";
import {ActivatedRoute} from "@angular/router";
import {
  AccessOrganizationalUnit,
  Institution,
  Role,
} from "@app/generated-api";
import {
  Select,
  Store,
} from "@ngxs/store";
import {SharedAbstractEditRoutable} from "@shared/components/routables/shared-abstract-edit/shared-abstract-edit.routable";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {LocalStateModel} from "@shared/models/local-state.model";
import {Observable} from "rxjs";

@Component({
  selector: "dlcm-admin-person-edit-routable",
  templateUrl: "./admin-person-edit.routable.html",
  styleUrls: ["../../../../../../shared/components/routables/shared-abstract-edit/shared-abstract-edit.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AdminPersonEditRoutable extends SharedAbstractEditRoutable<PersonExtended, AdminPersonStateModel> {
  @Select(AdminPersonState.isLoadingWithDependency) isLoadingWithDependencyObs: Observable<boolean>;
  @Select(AdminPersonState.isReadyToBeDisplayed) isReadyToBeDisplayedObs: Observable<boolean>;
  @Select((state: LocalStateModel) => state.shared.shared_institution.list) listInstitutionsObs: Observable<Institution[]>;
  @Select((state: LocalStateModel) => state.shared.shared_organizationalUnit.list) listOrgUnitObs: Observable<AccessOrganizationalUnit[]>;
  @Select((state: LocalStateModel) => state.admin.admin_person.admin_person_institutions.selected) selectedInstitutionsObs: Observable<Institution[]>;
  @Select((state: LocalStateModel) => state.shared.shared_role.list) listRoleObs: Observable<Role[]>;
  @Select((state: LocalStateModel) => state.admin.admin_person.admin_person_organizationalUnitRole.selected) selectedOrgUnitRoleObs: Observable<OrganizationalUnitRole[]>;

  constructor(protected store: Store,
              protected route: ActivatedRoute) {
    super(store, route, LocalStateEnum.admin_person, adminPersonActionNameSpace, LocalStateEnum.admin);
  }

  getSubResourceWithParentId(id: string): void {
    this.store.dispatch(new AdminPersonOrgUnitRoleAction.GetAll(id));
    this.store.dispatch(new AdminPersonInstitutionsAction.GetAll(id));
  }
}
