import {AdminPersonCreateRoutable} from "@admin/person/components/routables/admin-person-create/admin-person-create.routable";
import {AdminPersonDetailRoutable} from "@admin/person/components/routables/admin-person-detail/admin-person-detail.routable";
import {AdminPersonEditRoutable} from "@admin/person/components/routables/admin-person-edit/admin-person-edit.routable";
import {AdminPersonListRoutable} from "@admin/person/components/routables/admin-person-list/admin-person-list.routable";
import {NgModule} from "@angular/core";
import {
  RouterModule,
  Routes,
} from "@angular/router";
import {ApplicationRolePermissionEnum} from "@shared/enums/application-role-permission.enum";
import {
  AdminRoutesEnum,
  AppRoutesEnum,
} from "@shared/enums/routes.enum";
import {ApplicationRoleGuardService} from "@shared/guards/application-role-guard.service";
import {CanDeactivateGuard} from "@shared/services/can-deactivate-guard.service";
import {TRANSLATE} from "solidify-frontend";

const routes: Routes = [
  {
    path: AppRoutesEnum.root,
    component: AdminPersonListRoutable,
    data: {
      permission: ApplicationRolePermissionEnum.rootPermission,
    },
    canActivate: [ApplicationRoleGuardService],
  },
  {
    path: AdminRoutesEnum.personDetail + AppRoutesEnum.separator + AppRoutesEnum.paramId,
    component: AdminPersonDetailRoutable,
    data: {
      breadcrumb: TRANSLATE("breadcrumb.admin.person.detail"),
      permission: ApplicationRolePermissionEnum.rootPermission,
    },
  },
  {
    path: AdminRoutesEnum.personCreate,
    component: AdminPersonCreateRoutable,
    data: {
      breadcrumb: TRANSLATE("breadcrumb.admin.person.create"),
    },
    canDeactivate: [CanDeactivateGuard],
  },
  {
    path: AdminRoutesEnum.personEdit + AppRoutesEnum.separator + AppRoutesEnum.paramId,
    component: AdminPersonEditRoutable,
    data: {
      breadcrumb: TRANSLATE("breadcrumb.admin.person.edit"),
    },
    canDeactivate: [CanDeactivateGuard],
  },

];

@NgModule({
  imports: [RouterModule.forChild(routes)],
  exports: [RouterModule],
})
export class AdminPersonRoutingModule {
}
