import {PreservationPolicy} from "@app/generated-api";
import {
  Actions,
  Selector,
  State,
  Store,
} from "@ngxs/store";
import {AdminResourceApiEnum} from "@shared/enums/api.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {
  ApiService,
  defaultResourceStateInitValue,
  isNullOrUndefined,
  NotificationService,
  ResourceState,
  ResourceStateModel,
  StoreUtil,
  TRANSLATE,
} from "solidify-frontend";
import {adminPreservationPolicyActionNameSpace} from "./admin-preservation-policy.action";

export interface AdminPreservationPolicyStateModel extends ResourceStateModel<PreservationPolicy> {
}

@State<AdminPreservationPolicyStateModel>({
  name: LocalStateEnum.admin_preservationPolicy,
  defaults: {
    ...defaultResourceStateInitValue,
  },
})
export class AdminPreservationPolicyState extends ResourceState<PreservationPolicy> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: adminPreservationPolicyActionNameSpace,
      routeRedirectUrlAfterSuccessAction: RoutesEnum.adminPreservationPolicy,
      notificationResourceCreateTextToTranslate: TRANSLATE("admin.preservationPolicy.notification.resource.create"),
      notificationResourceDeleteTextToTranslate: TRANSLATE("admin.preservationPolicy.notification.resource.delete"),
      notificationResourceUpdateTextToTranslate: TRANSLATE("admin.preservationPolicy.notification.resource.update"),
    });
  }

  protected get _urlResource(): string {
    return AdminResourceApiEnum.preservationPolicies;
  }

  @Selector()
  static isLoading(state: AdminPreservationPolicyStateModel): boolean {
    return StoreUtil.isLoadingState(state);
  }

  @Selector()
  static isLoadingWithDependency(state: AdminPreservationPolicyStateModel): boolean {
    return this.isLoading(state);
  }

  @Selector()
  static isReadyToBeDisplayed(state: AdminPreservationPolicyStateModel): boolean {
    return this.isReadyToBeDisplayedInCreateMode
      && !isNullOrUndefined(state.current);
  }

  @Selector()
  static isReadyToBeDisplayedInCreateMode(state: AdminPreservationPolicyStateModel): boolean {
    return true;
  }
}
