import {adminRoleActionNameSpace} from "@admin/role/stores/admin-role.action";
import {Role} from "@app/generated-api";
import {
  Actions,
  Selector,
  State,
  Store,
} from "@ngxs/store";
import {AdminResourceApiEnum} from "@shared/enums/api.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {
  ApiService,
  defaultResourceStateInitValue,
  isNullOrUndefined,
  NotificationService,
  ResourceState,
  ResourceStateModel,
  StoreUtil,
  TRANSLATE,
} from "solidify-frontend";

export interface AdminRoleStateModel extends ResourceStateModel<Role> {
}

@State<AdminRoleStateModel>({
  name: LocalStateEnum.admin_role,
  defaults: {
    ...defaultResourceStateInitValue,
  },

})
export class AdminRoleState extends ResourceState<Role> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: adminRoleActionNameSpace,
      routeRedirectUrlAfterSuccessAction: RoutesEnum.admin,
      notificationResourceCreateTextToTranslate: TRANSLATE("admin.roles.notification.resource.create"),
      notificationResourceDeleteTextToTranslate: TRANSLATE("admin.roles.notification.resource.delete"),
      notificationResourceUpdateTextToTranslate: TRANSLATE("admin.roles.notification.resource.update"),
    });
  }

  protected get _urlResource(): string {
    return AdminResourceApiEnum.roles;
  }

  @Selector()
  static isLoading(state: AdminRoleStateModel): boolean {
    return StoreUtil.isLoadingState(state);
  }

  @Selector()
  static isLoadingWithDependency(state: AdminRoleStateModel): boolean {
    return this.isLoading(state);
  }

  @Selector()
  static isReadyToBeDisplayed(state: AdminRoleStateModel): boolean {
    return this.isReadyToBeDisplayedInCreateMode
      && !isNullOrUndefined(state.current);
  }

  @Selector()
  static isReadyToBeDisplayedInCreateMode(state: AdminRoleStateModel): boolean {
    return true;
  }
}
