import {SubmissionPolicy} from "@app/generated-api";
import {
  Actions,
  Selector,
  State,
  Store,
} from "@ngxs/store";
import {AdminResourceApiEnum} from "@shared/enums/api.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {
  ApiService,
  defaultResourceStateInitValue,
  isNullOrUndefined,
  NotificationService,
  ResourceState,
  ResourceStateModel,
  StoreUtil,
  TRANSLATE,
} from "solidify-frontend";
import {adminSubmissionPolicyActionNameSpace} from "./admin-submission-policy.action";

export interface AdminSubmissionPolicyStateModel extends ResourceStateModel<SubmissionPolicy> {
}

@State<AdminSubmissionPolicyStateModel>({
  name: LocalStateEnum.admin_submissionPolicy,
  defaults: {
    ...defaultResourceStateInitValue,
  },
})
export class AdminSubmissionPolicyState extends ResourceState<SubmissionPolicy> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: adminSubmissionPolicyActionNameSpace,
      routeRedirectUrlAfterSuccessAction: RoutesEnum.adminSubmissionPolicy,
      notificationResourceCreateTextToTranslate: TRANSLATE("admin.submissionPolicy.notification.resource.create"),
      notificationResourceDeleteTextToTranslate: TRANSLATE("admin.submissionPolicy.notification.resource.delete"),
      notificationResourceUpdateTextToTranslate: TRANSLATE("admin.submissionPolicy.notification.resource.update"),
    });
  }

  protected get _urlResource(): string {
    return AdminResourceApiEnum.submissionPolicies;
  }

  @Selector()
  static isLoading(state: AdminSubmissionPolicyStateModel): boolean {
    return StoreUtil.isLoadingState(state);
  }

  @Selector()
  static isLoadingWithDependency(state: AdminSubmissionPolicyStateModel): boolean {
    return this.isLoading(state);
  }

  @Selector()
  static isReadyToBeDisplayed(state: AdminSubmissionPolicyStateModel): boolean {
    return this.isReadyToBeDisplayedInCreateMode
      && !isNullOrUndefined(state.current);
  }

  @Selector()
  static isReadyToBeDisplayedInCreateMode(state: AdminSubmissionPolicyStateModel): boolean {
    return true;
  }
}
