import {
  AdminUserAction,
  adminUserActionNameSpace,
} from "@admin/user/stores/admin-user.action";
import {
  Action,
  Actions,
  ofActionCompleted,
  Selector,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {AdminResourceApiEnum} from "@shared/enums/api.enum";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {UserExtended} from "@shared/models/business/user-extended.model";
import {SharedPersonAction} from "@shared/stores/person/shared-person.action";
import {
  ApiService,
  defaultResourceStateInitValue,
  isNullOrUndefined,
  NotificationService,
  ResourceState,
  ResourceStateModel,
  StoreUtil,
  TRANSLATE,
} from "solidify-frontend";

export interface AdminUserStateModel extends ResourceStateModel<UserExtended> {
}

@State<AdminUserStateModel>({
  name: LocalStateEnum.admin_user,
  defaults: {
    ...defaultResourceStateInitValue,
  },

})
export class AdminUserState extends ResourceState<UserExtended> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: adminUserActionNameSpace,
      routeRedirectUrlAfterSuccessAction: RoutesEnum.adminUser,
      notificationResourceCreateTextToTranslate: TRANSLATE("admin.user.notification.resource.create"),
      notificationResourceDeleteTextToTranslate: TRANSLATE("admin.user.notification.resource.delete"),
      notificationResourceUpdateTextToTranslate: TRANSLATE("admin.user.notification.resource.update"),
    });
  }

  protected get _urlResource(): string {
    return AdminResourceApiEnum.users;
  }

  @Selector()
  static isLoading(state: AdminUserStateModel): boolean {
    return StoreUtil.isLoadingState(state);
  }

  @Selector()
  static isLoadingWithDependency(state: AdminUserStateModel): boolean {
    return this.isLoading(state);
  }

  @Selector()
  static isReadyToBeDisplayed(state: AdminUserStateModel): boolean {
    return this.isReadyToBeDisplayedInCreateMode
      && !isNullOrUndefined(state.current);
  }

  @Selector()
  static isReadyToBeDisplayedInCreateMode(state: AdminUserStateModel): boolean {
    return true;
  }

  @Action(AdminUserAction.LoadResource)
  loadResource(ctx: StateContext<AdminUserStateModel>, action: AdminUserAction.LoadResource): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    StoreUtil.dispatchParallelActionAndWaitForSubActionsCompletion(ctx, [
      {
        action: new SharedPersonAction.GetAll(),
        subActionCompletions: [
          this.actions$.pipe(ofActionCompleted(SharedPersonAction.GetAllSuccess)),
          this.actions$.pipe(ofActionCompleted(SharedPersonAction.GetAllFail)),
        ],
      },
    ]).subscribe(success => {
      if (success) {
        ctx.dispatch(new AdminUserAction.LoadResourceSuccess(action));
      } else {
        ctx.dispatch(new AdminUserAction.LoadResourceFail(action));
      }
    });
  }
}
