import {
  ChangeDetectionStrategy,
  Component,
  Input,
  Output,
} from "@angular/core";
import {DepositDataFile} from "@app/features/deposit/models/deposit-data-file.model";
import {FileUploadWrapper} from "@app/features/deposit/models/file-upload-wrapper.model";
import {UploadFileStatus} from "@app/features/deposit/models/upload-file-status.model";
import {DepositDataFileAction} from "@app/features/deposit/stores/data-file/deposit-data-file.action";
import {DepositAction} from "@app/features/deposit/stores/deposit.action";
import {SharedAbstractContainer} from "@app/shared/components/containers/shared-abstract/shared-abstract.container";
import {LocalStateModel} from "@app/shared/models/local-state.model";
import {DepositDataFileStatusHistoryAction} from "@deposit/stores/data-file/status-history/deposit-data-file-status-history.action";
import {
  Select,
  Store,
} from "@ngxs/store";
import {SharedHistoryDialog} from "@shared/components/dialogs/shared-history/shared-history.dialog";
import {StatusHistoryDialog} from "@shared/models/status-history-dialog.model";
import {
  BehaviorSubject,
  Observable,
} from "rxjs";
import {
  ObservableUtil,
  StoreUtil,
} from "solidify-frontend";

@Component({
  selector: "dlcm-deposit-file-container",
  templateUrl: "./deposit-file.container.html",
  styleUrls: ["./deposit-file.container.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DepositFileContainer extends SharedAbstractContainer {
  @Select((state: LocalStateModel) => state.deposit.deposit_dataFile.list) listDataFileObs: Observable<DepositDataFile[]>;
  @Select((state: LocalStateModel) => StoreUtil.isLoadingState(state.deposit.deposit_dataFile)) isLoadingDataFileObs: Observable<boolean>;
  @Select((state: LocalStateModel) => state.deposit.uploadStatus) uploadStatusObs: Observable<UploadFileStatus[]>;
  @Select((state: LocalStateModel) => StoreUtil.isLoadingState(state.deposit.deposit_dataFile.deposit_dataFile_statusHistory)) isLoadingHistoryObs: Observable<boolean>;


  @Input()
  parentResId: string;

  @Input()
  readonly: boolean = false;

  private readonly _showHistoryBS: BehaviorSubject<DepositDataFile | undefined> = new BehaviorSubject<DepositDataFile | undefined>(undefined);
  @Output("showHistoryChange")
  readonly showHistoryObs: Observable<DepositDataFile | undefined> = ObservableUtil.asObservable(this._showHistoryBS);

  constructor(protected store: Store) {
    super();
  }

  upload($event: FileUploadWrapper): void {
    this.store.dispatch(new DepositAction.SendDataFile(this.parentResId, $event));
  }

  uploadArchive($event: FileUploadWrapper): void {
    this.store.dispatch(new DepositAction.SendDataFile(this.parentResId, $event, true));
  }

  retry($event: UploadFileStatus): void {
    this.store.dispatch(new DepositAction.RetrySendDataFile(this.parentResId, $event));
  }

  cancel($event: UploadFileStatus): void {
    this.store.dispatch(new DepositAction.MarkAsCancelDataFileSending(this.parentResId, $event));
  }

  delete($event: DepositDataFile): void {
    this.store.dispatch(new DepositDataFileAction.Delete(this.parentResId, $event));
  }

  resume($event: DepositDataFile): void {
    this.store.dispatch(new DepositDataFileAction.ResumeDataFile(this.parentResId, $event));
  }

  refresh(): void {
    this.store.dispatch(new DepositDataFileAction.GetAll(this.parentResId, undefined, true));
  }

  download($event: DepositDataFile): void {
    this.store.dispatch(new DepositDataFileAction.Download(this.parentResId, $event));
  }

  showHistory(depositDataFile: DepositDataFile): void {
    this._showHistoryBS.next(depositDataFile);
  }
}
