import {
  ChangeDetectionStrategy,
  Component,
  OnInit,
} from "@angular/core";
import {
  FormBuilder,
  FormGroup,
  Validators,
} from "@angular/forms";
import {MatDialogRef} from "@angular/material";
import {FileUploadWrapper} from "@app/features/deposit/models/file-upload-wrapper.model";
import {SharedAbstractContainer} from "@app/shared/components/containers/shared-abstract/shared-abstract.container";
import {DataCategoryEnum} from "@app/shared/enums/data-category.enum";
import {DataTypeEnum} from "@app/shared/enums/data-type.enum";
import {DataCategoryHelper} from "@app/shared/helpers/data-category.helper";
import {BaseFormDefinition} from "@app/shared/models/base-form-definition.model";
import {PropertyName} from "solidify-frontend";

@Component({
  selector: "dlcm-deposit-file-upload-archive-dialog",
  templateUrl: "./deposit-file-upload-archive.dialog.html",
  styleUrls: ["./deposit-file-upload-archive.dialog.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DepositFileUploadArchiveDialog extends SharedAbstractContainer implements OnInit {
  form: FormGroup;
  formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();
  archiveToUpload: File;

  get dataCategoryHelper(): typeof DataCategoryHelper {
    return DataCategoryHelper;
  }

  readonly files: string = "files";

  constructor(protected _dialogRef: MatDialogRef<DepositFileUploadArchiveDialog>,
              protected _fb: FormBuilder) {
    super();
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.form = this._fb.group({
      [this.formDefinition.dataCategory]: [DataCategoryEnum.Primary, [Validators.required]],
      [this.formDefinition.dataType]: [DataTypeEnum.Reference, [Validators.required]],
    });
  }

  close(): void {
    this._dialogRef.close();
  }

  onFileChange(event: Event & any): void {
    if (event.target.files.length > 0) {
      const files: FileList = event.target.files;
      this.archiveToUpload = files[0];
    }
  }

  convertFileListToArray(files: FileList): File[] {
    const arrayFile: File[] = [];
    for (let i = 0; i < files.length; i++) {
      arrayFile.push(files.item(i));
    }
    return arrayFile;
  }

  getDate(lastModified: number): Date {
    return new Date(lastModified);
  }

  onSubmit(): void {
    const base = this.form.value as FileUploadWrapper;
    const fileUploadWrapper = {} as FileUploadWrapper;
    Object.assign(fileUploadWrapper, base);
    fileUploadWrapper.file = this.archiveToUpload;
    this._dialogRef.close(fileUploadWrapper);
  }

  getDataCategory(): DataCategoryEnum {
    return this.form.get(this.formDefinition.dataCategory).value;
  }

  delete(file: File): void {
    this.archiveToUpload = undefined;
  }
}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() dataCategory: string;
  @PropertyName() dataType: string;
}

