import {
  ChangeDetectionStrategy,
  Component,
  OnDestroy,
  OnInit,
  ViewChild,
} from "@angular/core";
import {MatTabGroup} from "@angular/material/tabs";
import {
  ActivatedRoute,
  NavigationCancel,
  NavigationEnd,
  Router,
} from "@angular/router";
import {DepositExtended} from "@app/features/deposit/models/deposits-extended.model";
import {DepositDataFileAction} from "@app/features/deposit/stores/data-file/deposit-data-file.action";
import {depositActionNameSpace} from "@app/features/deposit/stores/deposit.action";
import {
  DepositState,
  DepositStateModel,
} from "@app/features/deposit/stores/deposit.state";
import {DepositPersonAction} from "@app/features/deposit/stores/people/deposit-person.action";
import {SharedAbstractEditRoutable} from "@app/shared/components/routables/shared-abstract-edit/shared-abstract-edit.routable";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {SharedOrgUnitAction} from "@app/shared/stores/organizational-unit/shared-organizational-unit.action";
import {DepositTabEnum} from "@deposit/enums/deposit-tab.enum";
import {DepositHelper} from "@deposit/helpers/deposit.helper";
import {Navigate} from "@ngxs/router-plugin";
import {
  Select,
  Store,
} from "@ngxs/store";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {Observable} from "rxjs";
import {
  distinctUntilChanged,
  filter,
  tap,
} from "rxjs/operators";

@Component({
  selector: "dlcm-deposit-edit-routable",
  templateUrl: "./deposit-edit.routable.html",
  styleUrls: ["./deposit-edit.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DepositEditRoutable extends SharedAbstractEditRoutable<DepositExtended, DepositStateModel> implements OnInit, OnDestroy {
  @Select(DepositState.isLoadingWithDependency) isLoadingWithDependencyObs: Observable<boolean>;

  @ViewChild("matTabGroup", {static: false})
  readonly matTabGroup: MatTabGroup;

  tabSelected: DepositTabEnum;

  constructor(protected store: Store,
              protected route: ActivatedRoute,
              protected router: Router) {
    super(store, route, LocalStateEnum.deposit, depositActionNameSpace);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.store.dispatch(new SharedOrgUnitAction.GetAll());

    this._computeCurrentTab();
    this.subscribe(this.router.events
      .pipe(
        filter(event => event instanceof NavigationCancel || event instanceof NavigationEnd),
        distinctUntilChanged(),
        tap(event => {
          this._computeCurrentTab();
          this.matTabGroup.selectedIndex = this.tabSelected;
        }),
      ));
  }

  private _computeCurrentTab(): void {
    const tabRouteSelected = DepositHelper.getTabRouteSelected(this.route);
    this.tabSelected = DepositHelper.getTabSelectedIndexWithRoute(tabRouteSelected);
  }

  getSubResourceWithParentId(id: string): void {
    this.store.dispatch(new DepositDataFileAction.GetAll(id));
    this.store.dispatch(new DepositPersonAction.GetAll(id));
  }

  navigate(tabSelected: DepositTabEnum): void {
    const tabRouteSelected = DepositHelper.getTabRouteSelectedWithTabIndex(tabSelected);
    this.store.dispatch(new Navigate([RoutesEnum.depositEdit, this._resId, tabRouteSelected]));
  }

  backToDetail(): void {
    const tabRouteSelected = DepositHelper.getTabRouteSelectedWithTabIndex(this.tabSelected);
    this.store.dispatch(new Navigate([RoutesEnum.depositDetail, this._resId, tabRouteSelected]));
  }
}
