import {
  ChangeDetectionStrategy,
  Component,
  OnInit,
  ViewChild,
} from "@angular/core";
import {MatDialog} from "@angular/material";
import {ActivatedRoute} from "@angular/router";
import {DepositFormPresentational} from "@deposit/components/presentationals/deposit-form/deposit-form.presentational";
import {DepositHelper} from "@deposit/helpers/deposit.helper";
import {DepositDataFile} from "@deposit/models/deposit-data-file.model";
import {DepositExtended} from "@deposit/models/deposits-extended.model";
import {DepositDataFileAction} from "@deposit/stores/data-file/deposit-data-file.action";
import {DepositDataFileStatusHistoryAction} from "@deposit/stores/data-file/status-history/deposit-data-file-status-history.action";
import {depositActionNameSpace} from "@deposit/stores/deposit.action";
import {DepositStateModel} from "@deposit/stores/deposit.state";
import {
  Select,
  Store,
} from "@ngxs/store";
import {SharedHistoryDialog} from "@shared/components/dialogs/shared-history/shared-history.dialog";
import {SharedAbstractEditRoutable} from "@shared/components/routables/shared-abstract-edit/shared-abstract-edit.routable";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {LocalStateModel} from "@shared/models/local-state.model";
import {StatusHistoryDialog} from "@shared/models/status-history-dialog.model";
import {StatusHistory} from "@shared/models/status-history.model";
import {SharedOrgUnitAction} from "@shared/stores/organizational-unit/shared-organizational-unit.action";
import {Observable} from "rxjs";
import {ResourceActionHelper} from "solidify-frontend";

@Component({
  selector: "dlcm-deposit-file-routable",
  templateUrl: "./deposit-file.routable.html",
  styleUrls: ["./deposit-file.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DepositFileRoutable extends SharedAbstractEditRoutable<DepositExtended, DepositStateModel> implements OnInit {
  @Select((state: LocalStateModel) => state.deposit.deposit_dataFile.deposit_dataFile_statusHistory.history) historyObs: Observable<StatusHistory[]>;

  @ViewChild("formPresentational", {static: false})
  readonly formPresentational: DepositFormPresentational;

  isReadonly: boolean = true;

  constructor(protected store: Store,
              protected route: ActivatedRoute,
              public dialog: MatDialog) {
    super(store, route, LocalStateEnum.deposit, depositActionNameSpace, null, {
      skipGetData: true,
      skipClearOnDestroy: true,
    });
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.getCurrentModelOnParent();
    this.store.dispatch(new SharedOrgUnitAction.GetAll());
    this.isReadonly = DepositHelper.computeIsReadonly(this.route);
  }

  private getCurrentModelOnParent(): void {
    this._resId = this.route.snapshot.parent.paramMap.get("id");
    this.getDepositById(this._resId);
  }

  private getDepositById(id: string): void {
    this.store.dispatch(ResourceActionHelper.getById(depositActionNameSpace, id, true));
    this.getSubResourceWithParentId(id);
  }

  getSubResourceWithParentId(id: string): void {
    this.store.dispatch(new DepositDataFileAction.GetAll(id, undefined, true));
  }

  showHistory(depositDataFile: DepositDataFile): void {
    this.store.dispatch(new DepositDataFileStatusHistoryAction.History(depositDataFile.resId, this.getResId()));
    const dialogRef = this.dialog.open(SharedHistoryDialog, {
      width: "800px",
      data: {
        resourceResId: this._resId,
        name: this.state,
        statusHistory: this.historyObs,
        isLoading: this.store.select((s: LocalStateModel) => s.deposit.deposit_dataFile.deposit_dataFile_statusHistory.isLoadingHistoryCounter > 0),
      } as StatusHistoryDialog,
    });
  }

}
