import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnInit,
} from "@angular/core";
import {DepositExtended} from "@app/features/deposit/models/deposits-extended.model";
import {depositActionNameSpace} from "@app/features/deposit/stores/deposit.action";
import {DepositStateModel} from "@app/features/deposit/stores/deposit.state";
import {SharedAbstractListRoutable} from "@app/shared/components/routables/shared-abstract-list/shared-abstract-list.routable";
import {FieldTypeEnum} from "@app/shared/enums/field-type.enum";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {TranslateService} from "@ngx-translate/core";
import {Store} from "@ngxs/store";
import {
  OrderEnum,
  TRANSLATE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-deposit-list-routable",
  templateUrl: "../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.html",
  styleUrls: ["../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DepositListRoutable extends SharedAbstractListRoutable<DepositExtended, DepositStateModel> implements OnInit {
  readonly KEY_CREATE_BUTTON: string = TRANSLATE("deposit.new");
  readonly KEY_REFRESH_BUTTON: string = TRANSLATE("deposit.refresh");

  constructor(protected store: Store,
              protected _changeDetector: ChangeDetectorRef,
              private translate: TranslateService) {
    super(store, _changeDetector, LocalStateEnum.deposit, depositActionNameSpace);
  }

  ngOnInit(): void {
    super.ngOnInit();
  }

  defineColumns(): void {
    this.columns = [
      {
        field: "title",
        header: TRANSLATE("deposit.table.header.title"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "publicationDate",
        header: TRANSLATE("deposit.table.header.publicationDate"),
        type: FieldTypeEnum.date,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "creation.when" as any,
        header: TRANSLATE("deposit.table.header.creation.when"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "lastUpdate.when" as any,
        header: TRANSLATE("deposit.table.header.lastUpdate.when"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.descending,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "status",
        header: TRANSLATE("deposit.table.header.status"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
    ];
  }
}
