import {NgModule} from "@angular/core";
import {RouterModule} from "@angular/router";
import {DepositCreateRoutable} from "@app/features/deposit/components/routables/deposit-create/deposit-create.routable";
import {DepositDetailRoutable} from "@app/features/deposit/components/routables/deposit-detail/deposit-detail.routable";
import {DepositEditRoutable} from "@app/features/deposit/components/routables/deposit-edit/deposit-edit.routable";
import {DepositListRoutable} from "@app/features/deposit/components/routables/deposit-list/deposit-list.routable";
import {
  AppRoutesEnum,
  DepositRoutesEnum,
} from "@app/shared/enums/routes.enum";
import {DlcmRoutes} from "@app/shared/models/dlcm-route.model";
import {DepositFileRoutable} from "@deposit/components/routables/deposit-file/deposit-file.routable";
import {DepositMetadataRoutable} from "@deposit/components/routables/deposit-metadata/deposit-metadata.routable";
import {DepositRoleGuardService} from "@shared/guards/deposit-role-guard.service";
import {CanDeactivateGuard} from "@shared/services/can-deactivate-guard.service";
import {TRANSLATE} from "solidify-frontend";

const routes: DlcmRoutes = [
  {
    path: AppRoutesEnum.root,
    component: DepositListRoutable,
    data: {},
  },
  {
    path: DepositRoutesEnum.detail + AppRoutesEnum.separator + AppRoutesEnum.paramId,
    redirectTo: DepositRoutesEnum.detail + AppRoutesEnum.separator + AppRoutesEnum.paramId + AppRoutesEnum.separator + "file",
    pathMatch: "full",
  },
  {
    path: DepositRoutesEnum.detail + AppRoutesEnum.separator + AppRoutesEnum.paramId,
    component: DepositDetailRoutable,
    data: {
      breadcrumb: TRANSLATE("breadcrumb.deposit.detail"),
    },
    canActivate: [DepositRoleGuardService],
    children: [
      {
        path: DepositRoutesEnum.metadata,
        component: DepositMetadataRoutable,
        data: {
          breadcrumb: TRANSLATE("breadcrumb.deposit.metadata"),
        },
        canDeactivate: [CanDeactivateGuard],
      },
      {
        path: DepositRoutesEnum.file,
        component: DepositFileRoutable,
        data: {
          breadcrumb: TRANSLATE("breadcrumb.deposit.file"),
        },
        canDeactivate: [CanDeactivateGuard],
      },
    ],
  },
  {
    path: DepositRoutesEnum.edit + AppRoutesEnum.separator + AppRoutesEnum.paramId,
    redirectTo: DepositRoutesEnum.edit + AppRoutesEnum.separator + AppRoutesEnum.paramId + AppRoutesEnum.separator + "file",
    pathMatch: "full",
  },
  {
    path: DepositRoutesEnum.edit + AppRoutesEnum.separator + AppRoutesEnum.paramId,
    component: DepositEditRoutable,
    data: {
      breadcrumb: TRANSLATE("breadcrumb.deposit.edit"),
    },
    canActivate: [DepositRoleGuardService],
    children: [
      {
        path: DepositRoutesEnum.metadata,
        component: DepositMetadataRoutable,
        data: {
          breadcrumb: TRANSLATE("breadcrumb.deposit.metadata"),
        },
        canDeactivate: [CanDeactivateGuard],
      },
      {
        path: DepositRoutesEnum.file,
        component: DepositFileRoutable,
        data: {
          breadcrumb: TRANSLATE("breadcrumb.deposit.file"),
        },
        canDeactivate: [CanDeactivateGuard],
      },
    ],
  },
  {
    path: DepositRoutesEnum.create,
    component: DepositCreateRoutable,
    data: {
      breadcrumb: TRANSLATE("breadcrumb.deposit.create"),
    },
    canDeactivate: [CanDeactivateGuard],
  },
];

@NgModule({
  imports: [RouterModule.forChild(routes)],
  exports: [RouterModule],
})
export class DepositRoutingModule {
}
