import {
  HttpClient,
  HttpHeaders,
} from "@angular/common/http";
import {Inject} from "@angular/core";
import {WINDOW} from "@app/app.module";
import {DepositDataFile} from "@app/features/deposit/models/deposit-data-file.model";
import {
  DepositDataFileAction,
  depositDataFileActionNameSpace,
} from "@app/features/deposit/stores/data-file/deposit-data-file.action";
import {ApiActionEnum} from "@app/shared/enums/api-action.enum";
import {ApiResourceNameEnum} from "@app/shared/enums/api-resource-name.enum";
import {PreIngestResourceApiEnum} from "@app/shared/enums/api.enum";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {
  DepositDataFileStatusHistoryState,
  DepositDataFileStatusHistoryStateModel,
} from "@deposit/stores/data-file/status-history/deposit-data-file-status-history.state";
import {
  Action,
  Actions,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {saveAs} from "file-saver";
import {Observable} from "rxjs";
import {tap} from "rxjs/internal/operators/tap";
import {
  ApiService,
  CompositionState,
  CompositionStateModel,
  defaultCompositionStateInitValue,
  NotificationService,
  TRANSLATE,
} from "solidify-frontend";

export interface DepositDataFileStateModel extends CompositionStateModel<DepositDataFile> {
  deposit_dataFile_statusHistory: DepositDataFileStatusHistoryStateModel;
}

@State<DepositDataFileStateModel>({
  name: LocalStateEnum.deposit_dataFile,
  defaults: {
    ...defaultCompositionStateInitValue,
    deposit_dataFile_statusHistory: null,
  },
  children: [
    DepositDataFileStatusHistoryState,
  ],
})
export class DepositDataFileState extends CompositionState<DepositDataFile> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions,
              @Inject(WINDOW) private _window: Window,
              protected httpClient: HttpClient) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: depositDataFileActionNameSpace,
      resourceName: ApiResourceNameEnum.DATAFILE,
    });
  }

  protected get _urlResource(): string {
    return PreIngestResourceApiEnum.deposits;
  }

  @Action(DepositDataFileAction.ResumeDataFile)
  resume(ctx: StateContext<DepositDataFileState>, action: DepositDataFileAction.ResumeDataFile): Observable<void> {
    return this.apiService.post<void>(`${this._urlResource}/${action.parentId}/${ApiResourceNameEnum.DATAFILE}/${action.dataFile.resId}/${ApiActionEnum.RESUME}`)
      .pipe(
        tap(() => {
          this.notificationService.showInformation(TRANSLATE("notification.deposit.file.resumed"), true);
          ctx.dispatch(new DepositDataFileAction.GetAll(action.parentId, undefined, true));
        }),
      );
  }

  @Action(DepositDataFileAction.Download)
  download(ctx: StateContext<DepositDataFileState>, action: DepositDataFileAction.Download): void {
    let headers = new HttpHeaders();
    headers = headers.set("Content-Disposition", "attachment; filename=" + action.dataFile.fileName);

    this.httpClient.get(`${this._urlResource}/${action.parentId}/${ApiResourceNameEnum.DATAFILE}/${action.dataFile.resId}/${ApiActionEnum.DL}`, {
      headers,
      responseType: "blob",
    }).subscribe((res: Blob) => {
      this.saveFile(res, action.dataFile.fileName);
    });

    this.notificationService.showInformation(TRANSLATE("notification.deposit.file.download"), true);
  }

  saveFile(blobContent: Blob, fileName: string): void {
    const blob = new Blob([blobContent], {type: "application/octet-stream"});
    saveAs(blob, fileName);
  }
}
