import {DepositDataFile} from "@app/features/deposit/models/deposit-data-file.model";
import {DepositExtended} from "@app/features/deposit/models/deposits-extended.model";
import {FileUploadWrapper} from "@app/features/deposit/models/file-upload-wrapper.model";
import {UploadFileStatus} from "@app/features/deposit/models/upload-file-status.model";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {
  ErrorDto,
  ResourceAction,
  ResourceNameSpace,
  TypeDefaultAction,
} from "solidify-frontend";

const state = LocalStateEnum.deposit;

export namespace DepositAction {
  @TypeDefaultAction(state)
  export class LoadResource extends ResourceAction.LoadResource {
  }

  @TypeDefaultAction(state)
  export class LoadResourceSuccess extends ResourceAction.LoadResourceSuccess {
  }

  @TypeDefaultAction(state)
  export class LoadResourceFail extends ResourceAction.LoadResourceFail {
  }

  @TypeDefaultAction(state)
  export class ChangeQueryParameters extends ResourceAction.ChangeQueryParameters {
  }

  @TypeDefaultAction(state)
  export class GetAll extends ResourceAction.GetAll {
  }

  @TypeDefaultAction(state)
  export class GetAllSuccess extends ResourceAction.GetAllSuccess<DepositExtended> {
  }

  @TypeDefaultAction(state)
  export class GetAllFail extends ResourceAction.GetAllFail<DepositExtended> {
  }

  @TypeDefaultAction(state)
  export class GetById extends ResourceAction.GetById {
  }

  @TypeDefaultAction(state)
  export class GetByIdSuccess extends ResourceAction.GetByIdSuccess<DepositExtended> {
  }

  @TypeDefaultAction(state)
  export class GetByIdFail extends ResourceAction.GetByIdFail<DepositExtended> {
  }

  @TypeDefaultAction(state)
  export class Create extends ResourceAction.Create<DepositExtended> {
  }

  @TypeDefaultAction(state)
  export class CreateSuccess extends ResourceAction.CreateSuccess<DepositExtended> {
  }

  @TypeDefaultAction(state)
  export class CreateFail extends ResourceAction.CreateFail<DepositExtended> {
  }

  @TypeDefaultAction(state)
  export class Update extends ResourceAction.Update<DepositExtended> {
  }

  @TypeDefaultAction(state)
  export class UpdateSuccess extends ResourceAction.UpdateSuccess<DepositExtended> {
  }

  @TypeDefaultAction(state)
  export class UpdateFail extends ResourceAction.UpdateFail<DepositExtended> {
  }

  @TypeDefaultAction(state)
  export class Delete extends ResourceAction.Delete {
  }

  @TypeDefaultAction(state)
  export class DeleteSuccess extends ResourceAction.DeleteSuccess {
  }

  @TypeDefaultAction(state)
  export class DeleteFail extends ResourceAction.DeleteFail {
  }

  @TypeDefaultAction(state)
  export class Clean extends ResourceAction.Clean {
  }

  export class Submit {
    static readonly type: string = `[${state}] Submit`;

    constructor(public deposit: DepositExtended) {
    }
  }

  export class SubmitSuccess {
    static readonly type: string = `[${state}] Submit Success`;

    constructor(public deposit: DepositExtended) {
    }
  }

  export class SubmitFail {
    static readonly type: string = `[${state}] Submit Fail`;

    constructor() {
    }
  }

  export class Approve {
    static readonly type: string = `[${state}] Approve`;

    constructor(public deposit: DepositExtended) {
    }
  }

  export class ApproveSuccess {
    static readonly type: string = `[${state}] Approve Success`;

    constructor(public deposit: DepositExtended) {
    }
  }

  export class ApproveFail {
    static readonly type: string = `[${state}] Approve Fail`;

    constructor() {
    }
  }

  export class Reject {
    static readonly type: string = `[${state}] Reject`;

    constructor(public deposit: DepositExtended) {
    }
  }

  export class RejectSuccess {
    static readonly type: string = `[${state}] Reject Success`;

    constructor(public deposit: DepositExtended) {
    }
  }

  export class RejectFail {
    static readonly type: string = `[${state}] Reject Fail`;

    constructor() {
    }
  }

  export class BackToEdit {
    static readonly type: string = `[${state}] BackToEdit`;

    constructor(public deposit: DepositExtended) {
    }
  }

  export class BackToEditSuccess {
    static readonly type: string = `[${state}] BackToEdit Success`;

    constructor(public deposit: DepositExtended) {
    }
  }

  export class BackToEditFail {
    static readonly type: string = `[${state}] BackToEdit Fail`;

    constructor() {
    }
  }

  export class SendDataFile {
    static readonly type: string = `[${state}] Upload Data File`;

    constructor(public parentId: string, public fileUploadWrapper: FileUploadWrapper, public isArchive: boolean = false) {
    }
  }

  export class SendDataFileSuccess {
    static readonly type: string = `[${state}] Upload Data File Success`;

    constructor(public parentId: string, public uploadFileStatus: UploadFileStatus, public depositDataFile: DepositDataFile) {
    }
  }

  export class SendDataFileFail {
    static readonly type: string = `[${state}] Upload Data File Fail`;

    constructor(public uploadFileStatus: UploadFileStatus, public errorDto: ErrorDto | undefined) {
    }
  }

  export class RetrySendDataFile {
    static readonly type: string = `[${state}] Retry Send Data File`;

    constructor(public parentId: string, public uploadFileStatus: UploadFileStatus) {
    }
  }

  export class MarkAsCancelDataFileSending {
    static readonly type: string = `[${state}] Mark As Cancel Data File Sending`;

    constructor(public parentId: string, public uploadFileStatus: UploadFileStatus) {
    }
  }

  export class CancelDataFileSending {
    static readonly type: string = `[${state}] Cancel Data File Sending`;

    constructor(public parentId: string, public uploadFileStatus: UploadFileStatus) {
    }
  }
}

export const depositActionNameSpace: ResourceNameSpace = DepositAction;
