import {
  ChangeDetectionStrategy,
  Component,
  Input,
  OnInit,
  Output,
} from "@angular/core";
import {SearchScopeEnum} from "@app/features/home/enums/search-scope.enum";
import {AccessOrganizationalUnit} from "@app/generated-api";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {ArchiveMetadata} from "@app/shared/models/business/archive-metadata.model";
import {MetadataUtil} from "@app/shared/utils/metadata.util";
import {
  BehaviorSubject,
  Observable,
} from "rxjs";
import {
  DateUtil,
  ObservableUtil,
  TRANSLATE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-home-archive-detail",
  templateUrl: "./home-archive-detail.presentational.html",
  styleUrls: ["./home-archive-detail.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class HomeArchiveDetailPresentational extends SharedAbstractPresentational implements OnInit {
  listInfo: Info[];

  private readonly _downloadBS: BehaviorSubject<ArchiveMetadata | undefined> = new BehaviorSubject<ArchiveMetadata | undefined>(undefined);
  @Output("downloadChange")
  readonly downloadObs: Observable<ArchiveMetadata | undefined> = ObservableUtil.asObservable(this._downloadBS);

  @Input()
  archiveMetadata: ArchiveMetadata;

  @Input()
  searchScope: SearchScopeEnum;

  @Input()
  organizationalUnit: AccessOrganizationalUnit;

  @Input()
  isLoadingPrepareDownload: boolean;

  showCode: boolean = false;

  ngOnInit(): void {
    const metadata = this.archiveMetadata.metadata;
    this.listInfo = [
      {
        labelToTranslate: TRANSLATE("homePage.archive.detail.title"),
        value: MetadataUtil.getTitle(this.searchScope, metadata),
      },
      {
        labelToTranslate: TRANSLATE("homePage.archive.detail.description"),
        value: MetadataUtil.getDescription(this.searchScope, metadata),
      },
      {
        labelToTranslate: TRANSLATE("homePage.archive.detail.orgUnit"),
        value: this.organizationalUnit.name,
      },
      {
        labelToTranslate: TRANSLATE("homePage.archive.detail.authors"),
        value: MetadataUtil.getAuthors(this.searchScope, metadata),
      },
      {
        labelToTranslate: TRANSLATE("homePage.archive.detail.publicationYear"),
        value: MetadataUtil.getPublicationYear(this.searchScope, metadata),
      },
      {
        labelToTranslate: TRANSLATE("homePage.archive.detail.DOI"),
        value: MetadataUtil.getDOI(this.searchScope, metadata),
      },
      {
        labelToTranslate: TRANSLATE("homePage.archive.detail.licenses"),
        value: MetadataUtil.getLicenses(this.searchScope, metadata),
      },
      {
        labelToTranslate: TRANSLATE("homePage.archive.detail.accessLevel"),
        value: MetadataUtil.getAccessLevel(this.searchScope, metadata),
      },
      {
        labelToTranslate: TRANSLATE("homePage.archive.detail.embargoAccessLevel"),
        value: MetadataUtil.getEmbargoAccessLevel(this.searchScope, metadata),
      },
      {
        labelToTranslate: TRANSLATE("homePage.archive.detail.embargoEndDate"),
        value: DateUtil.convertDateToDateString(MetadataUtil.getEmbargoEndDate(this.searchScope, metadata)),
      },
      {
        labelToTranslate: TRANSLATE("homePage.archive.detail.submittedDate"),
        value: DateUtil.convertDateToDateTimeString(MetadataUtil.getSubmittedDate(this.searchScope, metadata)),
      },
      {
        labelToTranslate: TRANSLATE("homePage.archive.detail.acceptedDate"),
        value: DateUtil.convertDateToDateTimeString(MetadataUtil.getAcceptedDate(this.searchScope, metadata)),
      },
      {
        labelToTranslate: this.getMetadataLabel(),
        value: MetadataUtil.getDatacite(this.searchScope, this.archiveMetadata.metadata),
        code: true,
      },
    ];
  }

  download(archiveMetadata: ArchiveMetadata): void {
    this._downloadBS.next(archiveMetadata);
  }

  getMetadataLabel(): string {
    if (this.searchScope === SearchScopeEnum.public) {
      return TRANSLATE("homePage.archive.detail.datacite");
    } else {
      return TRANSLATE("homePage.archive.detail.dlcm");
    }
  }

  getMetadata(): string {
    return MetadataUtil.getDatacite(this.searchScope, this.archiveMetadata.metadata);
  }

}

class Info {
  labelToTranslate: string;
  value: string;
  code?: boolean = false;
}
