import {
  ChangeDetectionStrategy,
  Component,
  Input,
  Output,
} from "@angular/core";
import {SearchScopeEnum} from "@app/features/home/enums/search-scope.enum";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {ArchiveMetadata} from "@app/shared/models/business/archive-metadata.model";
import {MetadataUtil} from "@app/shared/utils/metadata.util";
import {
  BehaviorSubject,
  Observable,
} from "rxjs";
import {
  isNonEmptyString,
  ObservableUtil,
  StringUtil,
} from "solidify-frontend";
import PublicMetadata = ArchiveMetadata.PublicMetadata;

@Component({
  selector: "dlcm-home-browsing-archive-presentational",
  templateUrl: "./home-browsing-archive.presentational.html",
  styleUrls: ["./home-browsing-archive.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class HomeBrowsingArchivePresentational extends SharedAbstractPresentational {
  @Input()
  archive: ArchiveMetadata;

  private readonly _navigateDetailBS: BehaviorSubject<ArchiveMetadata | undefined> = new BehaviorSubject<ArchiveMetadata | undefined>(undefined);
  @Output("navigateDetailChange")
  readonly navigateDetailObs: Observable<ArchiveMetadata | undefined> = ObservableUtil.asObservable(this._navigateDetailBS);

  navigate(archive: ArchiveMetadata): void {
    this._navigateDetailBS.next(archive);
  }

  getTitle(archive: ArchiveMetadata): string {
    return MetadataUtil.getTitle(SearchScopeEnum.public, archive.metadata);
  }

  getPublicationYear(archive: ArchiveMetadata): string {
    return MetadataUtil.getPublicationYear(SearchScopeEnum.public, archive.metadata as PublicMetadata);
  }

  getAuthors(archive: ArchiveMetadata): string {
    return MetadataUtil.getAuthors(SearchScopeEnum.public, archive.metadata as PublicMetadata);
  }

  getDescription(archive: ArchiveMetadata): string {
    return MetadataUtil.getDescription(SearchScopeEnum.public, archive.metadata as PublicMetadata);
  }

  getLicenses(archive: ArchiveMetadata): string {
    return MetadataUtil.getLicenses(SearchScopeEnum.public, archive.metadata as PublicMetadata);
  }

  getContent(archive: ArchiveMetadata): string {
    const publicationYear = this.getPublicationYear(archive);
    const separator = " - ";
    const authors = this.getAuthors(archive);
    let content = StringUtil.stringEmpty;

    if (isNonEmptyString(publicationYear)) {
      content = publicationYear;
      if (isNonEmptyString(authors)) {
        content += separator + authors;
      } else {
        return content;
      }
    } else {
      if (isNonEmptyString(authors)) {
        return content = authors;
      }
    }
    return content;
  }
}
