import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  HostBinding,
  OnInit,
} from "@angular/core";
import {ActivatedRoute} from "@angular/router";
import {HomeOrgUnitArchiveAction} from "@app/features/home/stores/organizational-unit/archive/home-organizational-unit-archive.action";
import {SharedAbstractRoutable} from "@app/shared/components/routables/shared-abstract/shared-abstract.routable";
import {RoutesEnum} from "@app/shared/enums/routes.enum";
import {ArchiveMetadata} from "@app/shared/models/business/archive-metadata.model";
import {LocalStateModel} from "@app/shared/models/local-state.model";
import {Navigate} from "@ngxs/router-plugin";
import {
  Select,
  Store,
} from "@ngxs/store";
import {environment} from "environments/environment";
import {
  combineLatest,
  Observable,
} from "rxjs";
import {
  distinctUntilChanged,
  tap,
} from "rxjs/operators";
import {
  isEmptyArray,
  StoreUtil,
} from "solidify-frontend";
import {DlcmEnvironment} from "../../../../../../environments/environment.defaults.model";

@Component({
  selector: "dlcm-home-browsing-list-archives-routable",
  templateUrl: "./home-browsing-list-archives.routable.html",
  styleUrls: ["./home-browsing-list-archives.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class HomeBrowsingListArchivesRoutable extends SharedAbstractRoutable implements OnInit {
  @Select((state: LocalStateModel) => state.home.home_organizationalUnit.home_organizationalUnit_archive.selected) listObs: Observable<ArchiveMetadata[]>;
  @Select((state: LocalStateModel) => StoreUtil.isLoadingState(state.home.home_organizationalUnit.home_organizationalUnit_archive)) isLoadingObs: Observable<boolean>;

  @HostBinding("class.have-no-items") haveNoItems: boolean = true;

  get environment(): DlcmEnvironment {
    return environment;
  }

  constructor(private _route: ActivatedRoute,
              private _store: Store,
              private _cd: ChangeDetectorRef) {
    super();
  }

  ngOnInit(): void {
    super.ngOnInit();

    this.subscribe(this._route.params.pipe(
      tap(param => {
        const orgUnitId = param.id;
        this._store.dispatch(new HomeOrgUnitArchiveAction.GetAll(orgUnitId));
      }),
    ));

    this.subscribe(combineLatest(this.listObs, this.isLoadingObs).pipe(
      distinctUntilChanged(),
      tap(([list, isLoading]) => {
        if (isLoading) {
          this.haveNoItems = true;
          return;
        }
        this.haveNoItems = isEmptyArray(list);
      }),
    ));
  }

  navigateDetail(archive: ArchiveMetadata): void {
    this._store.dispatch(new Navigate([RoutesEnum.homePageDetail, archive.resId]));
  }

  close(): void {
    this._store.dispatch(new Navigate([RoutesEnum.homePageBrowsing]));
  }
}
