import {
  ChangeDetectionStrategy,
  Component,
  OnInit,
} from "@angular/core";
import {ActivatedRoute} from "@angular/router";
import {SearchScopeEnum} from "@app/features/home/enums/search-scope.enum";
import {HomeAction} from "@app/features/home/stores/home.action";
import {HomeState} from "@app/features/home/stores/home.state";
import {HomeOrgUnitAction} from "@app/features/home/stores/organizational-unit/home-organizational-unit.action";
import {AccessOrganizationalUnit} from "@app/generated-api";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {
  ArchiveMetadata,
  MetadataEnum,
} from "@app/shared/models/business/archive-metadata.model";
import {LocalStateModel} from "@app/shared/models/local-state.model";
import {
  Select,
  Store,
} from "@ngxs/store";
import {Observable} from "rxjs";
import {tap} from "rxjs/internal/operators/tap";
import {filter} from "rxjs/operators";
import {StoreUtil} from "solidify-frontend";
import {isNullOrUndefined} from "util";

@Component({
  selector: "dlcm-home-detail-routable",
  templateUrl: "./home-detail.routable.html",
  styleUrls: ["./home-detail.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class HomeDetailRoutable extends SharedAbstractPresentational implements OnInit {
  @Select(HomeState.isLoading) isLoadingObs: Observable<boolean>;
  @Select(HomeState.isLoadingPrepareDownload) isLoadingPrepareDownloadObs: Observable<boolean>;
  @Select(HomeState.current) currentObs: Observable<ArchiveMetadata>;
  @Select(HomeState.searchScope) searchScopeObs: Observable<SearchScopeEnum>;
  @Select((state: LocalStateModel) => state.home.home_organizationalUnit.current) organizationalUnitObs: Observable<AccessOrganizationalUnit>;
  @Select(HomeState) stateObs: AccessOrganizationalUnit;
  isLoadingOrgUnitObs: Observable<boolean>;

  private _resId: string;

  constructor(private store: Store,
              protected route: ActivatedRoute) {
    super();
    this.isLoadingOrgUnitObs = this.store.select((s: LocalStateModel) => StoreUtil.isLoadingState(s.home.home_organizationalUnit));
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.retrieveCurrentModelWithUrl();
    this.subscribe(this.currentObs.pipe(
      filter(m => !isNullOrUndefined(m)),
      tap(m => {
        this.store.dispatch(new HomeOrgUnitAction.GetById(m.metadata[MetadataEnum.aipOrganizationalUnit]));
      }),
    ));
  }

  private retrieveCurrentModelWithUrl(): void {
    this._resId = this.route.snapshot.paramMap.get("id");
    this.getById(this._resId);
  }

  private getById(id: string): void {
    this.store.dispatch(new HomeAction.SearchDetail(id));
  }

  download(archive: ArchiveMetadata): void {
    this.store.dispatch(new HomeAction.Download(archive));
  }
}
