import {SearchScopeEnum} from "@app/features/home/enums/search-scope.enum";
import {Rss} from "@app/features/home/models/rss.model";
import {ArchiveMetadata} from "@app/shared/models/business/archive-metadata.model";
import {
  CollectionTyped,
  QueryParameters,
} from "solidify-frontend";

export namespace HomeAction {
  export class ChangeQueryParameters {
    static readonly type: string = "[Home] Change Query Parameters";

    constructor(public queryParameters?: QueryParameters) {
    }
  }

  export class ChangeSearchScope {
    static readonly type: string = "[Home] Change Filter";

    constructor(public searchScope: SearchScopeEnum) {
    }

  }

  export class Search {
    static readonly type: string = "[Home] Search";

    constructor(public resetPagination: boolean, public search?: string, public queryParameters?: QueryParameters) {
    }
  }

  export class SearchSuccess {
    static readonly type: string = "[Home] Search Success";

    constructor(public list?: CollectionTyped<ArchiveMetadata> | null | undefined) {
    }
  }

  export class SearchFail {
    static readonly type: string = "[Home] Search Fail";
  }

  export class SearchDetail {
    static readonly type: string = "[Home] Search Detail";

    constructor(public resId: string) {
    }
  }

  export class SearchDetailSuccess {
    static readonly type: string = "[Home] Search Detail Success";

    constructor(public model: ArchiveMetadata) {
    }
  }

  export class SearchDetailFail {
    static readonly type: string = "[Home] Search Detail Fail";

    constructor() {
    }
  }

  export class GetRss {
    static readonly type: string = "[Home] Get Rss";

    constructor() {
    }
  }

  export class GetRssSuccess {
    static readonly type: string = "[Home] Get Rss Success";

    constructor(public rss: Rss) {
    }
  }

  export class GetRssFail {
    static readonly type: string = "[Home] Get Rss Fail";

    constructor() {
    }
  }

  export class Download {
    static readonly type: string = `[Home] Download`;

    constructor(public dataFile: ArchiveMetadata) {
    }
  }

  export class DownloadFail {
    static readonly type: string = `[Home] Download Fail`;

    constructor() {
    }
  }


}
