import {
  HomeOrgUnitArchiveAction,
  homeOrgUnitArchiveActionNameSpace,
} from "@app/features/home/stores/organizational-unit/archive/home-organizational-unit-archive.action";
import {ApiResourceNameEnum} from "@app/shared/enums/api-resource-name.enum";
import {AccessResourceApiEnum} from "@app/shared/enums/api.enum";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {ArchiveMetadata} from "@app/shared/models/business/archive-metadata.model";

import {
  Action,
  Actions,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {Observable} from "rxjs";
import {
  ApiService,
  AssociationNoSqlReadOnlyState,
  AssociationNoSqlReadOnlyStateModel,
  CollectionTyped,
  defaultAssociationNoSqlReadOnlyStateInitValue,
  NotificationService,
  OverrideDefaultAction,
} from "solidify-frontend";

export interface HomeOrganizationalUnitArchiveStateModel extends AssociationNoSqlReadOnlyStateModel<ArchiveMetadata> {
}

@State<HomeOrganizationalUnitArchiveStateModel>({
  name: LocalStateEnum.home_organizationalUnit_archive,
  defaults: {
    ...defaultAssociationNoSqlReadOnlyStateInitValue,
  },
})
export class HomeOrganizationalUnitArchiveState extends AssociationNoSqlReadOnlyState<ArchiveMetadata> {

  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: homeOrgUnitArchiveActionNameSpace,
      resourceName: ApiResourceNameEnum.ARCHIVE,
    });
  }

  protected get _urlResource(): string {
    return AccessResourceApiEnum.organizationalUnits;
  }

  @OverrideDefaultAction()
  @Action(HomeOrgUnitArchiveAction.GetAll)
  getAll<U>(ctx: StateContext<HomeOrganizationalUnitArchiveStateModel>, action: HomeOrgUnitArchiveAction.GetAll): Observable<CollectionTyped<U>> {
    ctx.patchState({selected: []});
    return super.getAll(ctx, action);
  }
}
