import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  Input,
} from "@angular/core";
import {
  FormBuilder,
  Validators,
} from "@angular/forms";
import {OrganizationalUnitExtended} from "@app/features/deposit/models/organizational-unit-extended.model";
import {
  PreservationPolicy,
  SubmissionPolicy,
} from "@app/generated-api";
import {SharedAbstractFormPresentational} from "@app/shared/components/presentationals/shared-abstract-form/shared-abstract-form.presentational";
import {BaseFormDefinition} from "@app/shared/models/base-form-definition.model";
import {BreakpointService} from "@shared/services/breakpoint.service";
import {
  DateUtil,
  PropertyName,
  SolidifyValidator,
} from "solidify-frontend";

@Component({
  selector: "dlcm-orgunit-form",
  templateUrl: "./orgunit-form.presentational.html",
  styleUrls: ["../../../../../shared/components/presentationals/shared-abstract-form/shared-abstract-form.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class OrgunitFormPresentational extends SharedAbstractFormPresentational<OrganizationalUnitExtended> {
  formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  @Input()
  listSubmissionPolicies: SubmissionPolicy[];

  @Input()
  listPreservationPolicies: PreservationPolicy[];

  constructor(protected readonly _changeDetectorRef: ChangeDetectorRef,
              private readonly _fb: FormBuilder,
              public readonly breakpointService: BreakpointService) {
    super(_changeDetectorRef);
  }

  protected initNewForm(): void {
    this.form = this._fb.group({
      [this.formDefinition.name]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.description]: ["", [SolidifyValidator]],
      [this.formDefinition.openingDate]: [""],
      [this.formDefinition.closingDate]: [""],
      [this.formDefinition.isEmpty]: [false],
      [this.formDefinition.open]: [false],
      [this.formDefinition.submissionPolicies]: [],
    });
  }

  protected bindFormTo(organizationalUnit: OrganizationalUnitExtended): void {
    this.form = this._fb.group({
      [this.formDefinition.name]: [organizationalUnit.name, [Validators.required, SolidifyValidator]],
      [this.formDefinition.description]: [organizationalUnit.description, [SolidifyValidator]],
      [this.formDefinition.openingDate]: [organizationalUnit.openingDate],
      [this.formDefinition.closingDate]: [organizationalUnit.closingDate],
      [this.formDefinition.isEmpty]: [organizationalUnit.isEmpty],
      [this.formDefinition.open]: [organizationalUnit.open],
    });
  }

  protected treatmentBeforeSubmit(organizationalUnit: OrganizationalUnitExtended): OrganizationalUnitExtended {
    organizationalUnit.openingDate = DateUtil.convertToLocalDateDateSimple(organizationalUnit.openingDate);
    organizationalUnit.closingDate = DateUtil.convertToLocalDateDateSimple(organizationalUnit.closingDate);
    return organizationalUnit;
  }
}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() name: string;
  @PropertyName() description: string;
  @PropertyName() isEmpty: string;
  @PropertyName() closingDate: string;
  @PropertyName() openingDate: string;
  @PropertyName() submissionPolicies: string;
  @PropertyName() open: string;
}
