import {
  ChangeDetectionStrategy,
  Component,
} from "@angular/core";
import {MatDialog} from "@angular/material";
import {ActivatedRoute} from "@angular/router";
import {OrganizationalUnitExtended} from "@app/features/deposit/models/organizational-unit-extended.model";
import {orgUnitActionNameSpace} from "@app/features/organizational-unit/stores/organizational-unit.action";
import {OrganizationalUnitStateModel} from "@app/features/organizational-unit/stores/organizational-unit.state";
import {
  PreservationPolicy,
  SubmissionPolicy,
} from "@app/generated-api";
import {SharedAbstractDetailRoutable} from "@app/shared/components/routables/shared-abstract-detail/shared-abstract-detail.routable";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {LocalStateModel} from "@app/shared/models/local-state.model";
import {
  Select,
  Store,
} from "@ngxs/store";
import {Observable} from "rxjs";
import {TRANSLATE} from "solidify-frontend";

@Component({
  selector: "dlcm-orgunit-detail-routable",
  templateUrl: "./orgunit-detail.routable.html",
  styleUrls: ["../../../../../shared/components/routables/shared-abstract-detail/shared-abstract-detail.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class OrgunitDetailRoutable extends SharedAbstractDetailRoutable<OrganizationalUnitExtended, OrganizationalUnitStateModel> {
  @Select((state: LocalStateModel) => state.shared.shared_submissionPolicy.list) listSubmissionPoliciesObs: Observable<SubmissionPolicy[]>;
  @Select((state: LocalStateModel) => state.shared.shared_preservationPolicy.list) listPreservationPoliciesObs: Observable<PreservationPolicy[]>;

  readonly KEY_PARAM_NAME: string = "name";
  readonly KEY_DELETE_BUTTON: string = TRANSLATE("organizationalUnit.delete");
  readonly KEY_EDIT_BUTTON: string = TRANSLATE("organizationalUnit.edit");

  constructor(protected store: Store,
              protected route: ActivatedRoute,
              public dialog: MatDialog) {
    super(store, route, dialog, LocalStateEnum.organizationalUnit, orgUnitActionNameSpace);
  }

  getSubResourceWithParentId(id: string): void {
  }
}
