import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
} from "@angular/core";
import {OrganizationalUnitExtended} from "@app/features/deposit/models/organizational-unit-extended.model";
import {orgUnitActionNameSpace} from "@app/features/organizational-unit/stores/organizational-unit.action";
import {OrganizationalUnitStateModel} from "@app/features/organizational-unit/stores/organizational-unit.state";
import {SharedAbstractListRoutable} from "@app/shared/components/routables/shared-abstract-list/shared-abstract-list.routable";
import {FieldTypeEnum} from "@app/shared/enums/field-type.enum";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {Store} from "@ngxs/store";
import {TRANSLATE} from "solidify-frontend";

@Component({
  selector: "dlcm-orgunit-list-routable",
  templateUrl: "../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.html",
  styleUrls: ["../../../../../shared/components/routables/shared-abstract-list/shared-abstract-list.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class OrgunitListRoutable extends SharedAbstractListRoutable<OrganizationalUnitExtended, OrganizationalUnitStateModel> {
  readonly KEY_CREATE_BUTTON: string = TRANSLATE("organizationalUnit.new");
  readonly KEY_REFRESH_BUTTON: string = TRANSLATE("organizationalUnit.refresh");

  constructor(protected store: Store,
              protected _changeDetector: ChangeDetectorRef) {
    super(store, _changeDetector, LocalStateEnum.organizationalUnit, orgUnitActionNameSpace, {canCreate: false});
  }

  defineColumns(): void {
    this.columns = [
      {
        field: "name",
        header: TRANSLATE("organizationalUnit.table.header.name"),
        type: FieldTypeEnum.string,
        order: 1,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "description",
        header: TRANSLATE("organizationalUnit.table.header.description"),
        type: FieldTypeEnum.string,
        order: 0,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "creation.when" as any,
        header: TRANSLATE("organizationalUnit.table.header.creation.when"),
        type: FieldTypeEnum.datetime,
        order: 0,
        isFilterable: true,
        isSortable: true,
      },
    ];
  }
}
