import {OrganizationalUnitExtended} from "@app/features/deposit/models/organizational-unit-extended.model";
import {orgUnitActionNameSpace} from "@app/features/organizational-unit/stores/organizational-unit.action";
import {AdminResourceApiEnum} from "@app/shared/enums/api.enum";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {RoutesEnum} from "@app/shared/enums/routes.enum";
import {
  Actions,
  State,
  Store,
} from "@ngxs/store";
import {
  ApiService,
  defaultResourceStateInitValue,
  NotificationService,
  ResourceState,
  ResourceStateModel,
  TRANSLATE,
} from "solidify-frontend";

export interface OrganizationalUnitStateModel extends ResourceStateModel<OrganizationalUnitExtended> {
}

@State<OrganizationalUnitStateModel>({
  name: LocalStateEnum.organizationalUnit,
  defaults: {
    ...defaultResourceStateInitValue,
  },
  children: [],
})
export class OrganizationalUnitState extends ResourceState<OrganizationalUnitExtended> {

  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: orgUnitActionNameSpace,
      routeRedirectUrlAfterSuccessAction: RoutesEnum.organizationalUnit,
      notificationResourceCreateTextToTranslate: TRANSLATE("organizationalUnit.notification.resource.create"),
      notificationResourceDeleteTextToTranslate: TRANSLATE("organizationalUnit.notification.resource.delete"),
      notificationResourceUpdateTextToTranslate: TRANSLATE("organizationalUnit.notification.resource.update"),
    });
  }

  protected get _urlResource(): string {
    return AdminResourceApiEnum.organizationalUnits;
  }
}
