import {
  ChangeDetectionStrategy,
  Component,
  Inject,
} from "@angular/core";
import {MAT_SNACK_BAR_DATA} from "@angular/material/snack-bar";
import {
  AbstractSnackbarPresentational,
  isNullOrUndefined,
  NotificationService,
  NotificationTypeEnum,
  SnackbarData,
} from "solidify-frontend";

@Component({
  selector: "dlcm-shared-snackbar",
  templateUrl: "./shared-snackbar.presentational.html",
  styleUrls: ["./shared-snackbar.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class SharedSnackbarPresentational extends AbstractSnackbarPresentational {
  get notificationTypeEnum(): typeof NotificationTypeEnum {
    return NotificationTypeEnum;
  }

  constructor(@Inject(MAT_SNACK_BAR_DATA) public data: SharedSnackbarOption, private snackbar: NotificationService) {
    super();
  }

  close(): void {
    this.snackbar.dismiss();
  }

  getIcon(): string | undefined {
    if (!isNullOrUndefined(this.data.icon)) {
      return this.data.icon;
    }

    switch (this.data.category) {
      case NotificationTypeEnum.success:
        return "check_circle";
      case NotificationTypeEnum.information:
        return "info";
      case NotificationTypeEnum.warning:
        return "warning";
      case NotificationTypeEnum.error:
        return "error";
    }

    return undefined;
  }
}

export interface SharedSnackbarOption extends SnackbarData {
  icon?: string;
}
