import {OnInit} from "@angular/core";
import {SharedAbstractCrudRoutable} from "@app/shared/components/routables/shared-abstract-crud/shared-abstract-crud.routable";
import {Store} from "@ngxs/store";
import {Observable} from "rxjs";
import {
  ModelFormControlEvent,
  ResourceActionHelper,
  ResourceNameSpace,
  ResourceStateModel,
  StoreUtil,
} from "solidify-frontend";
import {LocalStateEnum} from "../../../enums/local-state.enum";

export abstract class SharedAbstractCreateRoutable<TResourceModel, UResourceStateModel extends ResourceStateModel<TResourceModel>> extends SharedAbstractCrudRoutable<TResourceModel, UResourceStateModel> implements OnInit {
  isLoadingObs: Observable<boolean>;

  protected constructor(protected store: Store,
                        protected state: LocalStateEnum,
                        private resourceActionNameSpace: ResourceNameSpace,
                        protected parentState?: LocalStateEnum) {
    super(state, parentState);
    this.isLoadingObs = this.store.select(s => StoreUtil.isLoadingState(super.getState(s)));
  }

  create(modelFormControlEvent: ModelFormControlEvent<TResourceModel>): void {
    super.saveInProgress();
    this.store.dispatch(ResourceActionHelper.create(this.resourceActionNameSpace, modelFormControlEvent));
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.store.dispatch(ResourceActionHelper.loadResource(this.resourceActionNameSpace));
  }
}
