import {ViewChild} from "@angular/core";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {SharedAbstractFormPresentational} from "@shared/components/presentationals/shared-abstract-form/shared-abstract-form.presentational";
import {SharedAbstractResourceRoutable} from "@shared/components/routables/shared-abstract-resource/shared-abstract-resource.routable";
import {ICanDeactivate} from "@shared/models/can-deactivate.model";
import {
  isNullOrUndefined,
  ResourceStateModel,
} from "solidify-frontend";

export abstract class SharedAbstractCrudRoutable<TResourceModel, UResourceStateModel extends ResourceStateModel<TResourceModel>>
  extends SharedAbstractResourceRoutable<TResourceModel, UResourceStateModel>
  implements ICanDeactivate {
  @ViewChild("formPresentational", {static: false})
  readonly formPresentational: SharedAbstractFormPresentational<TResourceModel>;

  private _saveInProgressState: boolean = false;

  protected constructor(protected state: LocalStateEnum,
                        protected parentState?: LocalStateEnum) {
    super(state, parentState);
  }

  canDeactivate(): boolean {
    if (isNullOrUndefined(this.formPresentational) || isNullOrUndefined(this.formPresentational.form)) {
      return true;
    }
    return !this.formPresentational.form.dirty || this._saveInProgressState;
  }

  saveInProgress(): void {
    this._saveInProgressState = true;
  }
}
