import {ComponentType} from "@angular/cdk/portal";
import {
  OnDestroy,
  OnInit,
} from "@angular/core";
import {MatDialog} from "@angular/material";
import {ActivatedRoute} from "@angular/router";
import {SharedAbstractDeleteDialog} from "@app/shared/components/dialogs/shared-abstract-delete/shared-abstract-delete.dialog";
import {SharedAbstractCrudRoutable} from "@app/shared/components/routables/shared-abstract-crud/shared-abstract-crud.routable";
import {LocalModelAttributeEnum} from "@app/shared/enums/model-attribute.enum";
import {DeleteDialog} from "@app/shared/models/delete-dialog.model";
import {StoreDialogUtil} from "@app/shared/utils/store-dialog.util";
import {StoreRouteLocalUtil} from "@app/shared/utils/store-route-local.util";
import {Navigate} from "@ngxs/router-plugin";
import {Store} from "@ngxs/store";
import {Observable} from "rxjs";
import {take} from "rxjs/operators";
import {
  isFalse,
  isNullOrUndefined,
  ResourceActionHelper,
  ResourceNameSpace,
  ResourceStateModel,
  StoreUtil,
} from "solidify-frontend";
import {LocalStateEnum} from "../../../enums/local-state.enum";

export abstract class SharedAbstractDetailRoutable<TResourceModel, UResourceStateModel extends ResourceStateModel<TResourceModel>> extends SharedAbstractCrudRoutable<TResourceModel, UResourceStateModel> implements OnInit, OnDestroy {
  isLoadingObs: Observable<boolean>;
  currentObs: Observable<TResourceModel>;

  abstract readonly KEY_PARAM_NAME: string;
  abstract readonly KEY_EDIT_BUTTON: string;
  abstract readonly KEY_DELETE_BUTTON: string;

  protected _resId: string;

  protected constructor(protected store: Store,
                        protected route: ActivatedRoute,
                        public dialog: MatDialog,
                        protected state: LocalStateEnum,
                        private resourceNameSpace: ResourceNameSpace,
                        protected parentState?: LocalStateEnum,
                        protected option?: OptionSharedAbstractCreateRoutable) {
    super(state, parentState);
    this.isLoadingObs = this.store.select(s => StoreUtil.isLoadingState(super.getState(s)));
    this.currentObs = this.store.select(s => super.getState(s).current);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.store.dispatch(ResourceActionHelper.loadResource(this.resourceNameSpace));
    this.retrieveCurrentModelWithUrl();
  }

  ngOnDestroy(): void {
    super.ngOnDestroy();
    if (isNullOrUndefined(this.option) || isNullOrUndefined(this.option.skipClearOnDestroy) || isFalse(this.option.skipClearOnDestroy)) {
      this.store.dispatch(ResourceActionHelper.clean(this.resourceNameSpace));
    }
  }

  private retrieveCurrentModelWithUrl(): void {
    this._resId = this.route.snapshot.paramMap.get("id");
    this.getById(this._resId);
  }

  private getById(id: string): void {
    this.store.dispatch(ResourceActionHelper.getById(this.resourceNameSpace, id));
    this.getSubResourceWithParentId(id);
  }

  abstract getSubResourceWithParentId(id: string): void;

  getResId(): string {
    return this._resId;
  }

  edit(): void {
    this.store.dispatch(new Navigate([StoreRouteLocalUtil.getEditRoute(this.state), this._resId]));
  }

  delete(): void {
    const datas = {} as DeleteDialog;
    this.subscribe(
      this.currentObs.pipe(take(1)),
      (model: TResourceModel) => {
        datas.name = model[this.KEY_PARAM_NAME];
        datas.resId = model[LocalModelAttributeEnum.resId];
      });
    const dialogRef = this.dialog.open(StoreDialogUtil.delete(this.state) as ComponentType<SharedAbstractDeleteDialog<TResourceModel, UResourceStateModel>>, {
      width: "250px",
      data: datas,
    });
  }
}

export interface OptionSharedAbstractCreateRoutable {
  skipClearOnDestroy?: boolean;
}
