import {
  OnDestroy,
  OnInit,
  ViewChild,
} from "@angular/core";
import {ActivatedRoute} from "@angular/router";
import {SharedAbstractCrudRoutable} from "@app/shared/components/routables/shared-abstract-crud/shared-abstract-crud.routable";
import {Store} from "@ngxs/store";
import {SharedAbstractFormPresentational} from "@shared/components/presentationals/shared-abstract-form/shared-abstract-form.presentational";
import {Observable} from "rxjs";
import {
  isFalse,
  isNullOrUndefined,
  ModelFormControlEvent,
  ResourceActionHelper,
  ResourceNameSpace,
  ResourceStateModel,
  StoreUtil,
} from "solidify-frontend";
import {LocalStateEnum} from "../../../enums/local-state.enum";

export abstract class SharedAbstractEditRoutable<TResourceModel, UResourceStateModel extends ResourceStateModel<TResourceModel>> extends SharedAbstractCrudRoutable<TResourceModel, UResourceStateModel> implements OnInit, OnDestroy {
  isLoadingObs: Observable<boolean>;
  currentObs: Observable<TResourceModel>;

  @ViewChild("formPresentational", {static: false})
  readonly formPresentational: SharedAbstractFormPresentational<any>;

  protected _resId: string;

  protected constructor(protected store: Store,
                        protected route: ActivatedRoute,
                        protected state: LocalStateEnum,
                        private resourceNameSpace: ResourceNameSpace,
                        protected parentState?: LocalStateEnum,
                        protected option?: OptionSharedAbstractEditRoutable) {
    super(state, parentState);
    this.isLoadingObs = this.store.select(s => StoreUtil.isLoadingState(super.getState(s)));
    this.currentObs = this.store.select(s => super.getState(s).current);
  }

  ngOnInit(): void {
    super.ngOnInit();
    if (isNullOrUndefined(this.option) || !this.option.skipGetData) {
      this.store.dispatch(ResourceActionHelper.loadResource(this.resourceNameSpace));
      this.getCurrentModel();
    }
  }

  ngOnDestroy(): void {
    super.ngOnDestroy();
    if (isNullOrUndefined(this.option) || isNullOrUndefined(this.option.skipClearOnDestroy) || isFalse(this.option.skipClearOnDestroy)) {
      this.store.dispatch(ResourceActionHelper.clean(this.resourceNameSpace));
    }
  }

  private getCurrentModel(): void {
    this._resId = this.route.snapshot.paramMap.get("id");
    this.getById(this._resId);
  }

  private getById(id: string): void {
    this.store.dispatch(ResourceActionHelper.getById(this.resourceNameSpace, id));
    this.getSubResourceWithParentId(id);
  }

  abstract getSubResourceWithParentId(id: string): void;
  getResId(): string {
    return this._resId;
  }


  update(model: ModelFormControlEvent<TResourceModel>): void {
    super.saveInProgress();
    this.store.dispatch(ResourceActionHelper.update<TResourceModel>(this.resourceNameSpace, model));
  }
}

export interface OptionSharedAbstractEditRoutable {
  skipGetData?: boolean;
  skipClearOnDestroy?: boolean;
}
