import {ApiModuleEnum} from "@app/shared/enums/api-module.enum";
import {ApiResourceNameEnum} from "@app/shared/enums/api-resource-name.enum";
import {ResourceApiEnum} from "solidify-frontend";
import {environment} from "../../../environments/environment";

export class ApiKeyword {
  static PARENT_ID: string = "parentId";
  static SUB_RESOURCE: string = "subResource";
}

export const SEPARATOR: string = "/";
const subResource = `{${ApiKeyword.SUB_RESOURCE}}`;
const parentId = `{${ApiKeyword.PARENT_ID}}`;
const parentIdInUrl = SEPARATOR + parentId + SEPARATOR;
const search = "search";
const api = "api";
const apiInUrl = SEPARATOR + api + SEPARATOR;

const useEnvironmentApiUrl: boolean = environment.production === true || environment.forceRuntimeOverrideSetting === true;

export class BaseResourceApiEnum {
  static get access(): string {
    return (useEnvironmentApiUrl ? environment.ACCESS_APPLICATION : apiInUrl) + ApiModuleEnum.ACCESS;
  }

  static get admin(): string {
    return (useEnvironmentApiUrl ? environment.ADMIN_APPLICATION : apiInUrl) + ApiModuleEnum.ADMIN;
  }

  static get dataMgmt(): string {
    return (useEnvironmentApiUrl ? environment.ADMIN_APPLICATION : apiInUrl) + ApiModuleEnum.DATAMGMT;
  }

  static get resourceSrv(): string {
    return (useEnvironmentApiUrl ? environment.ADMIN_APPLICATION : apiInUrl) + ApiModuleEnum.RES_SRV;
  }

  static get preservationPlanning(): string {
    return (useEnvironmentApiUrl ? environment.ADMIN_APPLICATION : apiInUrl) + ApiModuleEnum.PRES_PLANNING;
  }

  static get ingest(): string {
    return (useEnvironmentApiUrl ? environment.INGEST_APPLICATION : apiInUrl) + ApiModuleEnum.INGEST;
  }

  static get preingest(): string {
    return (useEnvironmentApiUrl ? environment.INGEST_APPLICATION : apiInUrl) + ApiModuleEnum.PREINGEST;
  }

  static get archivalStorage(): string {
    return (useEnvironmentApiUrl ? environment.STORAGION_APPLICATION : apiInUrl) + ApiModuleEnum.ARCHIVALSTORAGE;
  }
}

export interface SubResourceApiEnum extends ResourceApiEnum {
}

export class AccessResourceApiEnum implements ResourceApiEnum {
  static get root(): string {
    return BaseResourceApiEnum.access;
  }

  static get dip(): string {
    return BaseResourceApiEnum.access + SEPARATOR + ApiResourceNameEnum.DIP;
  }

  static get allMetadata(): string {
    return BaseResourceApiEnum.access + SEPARATOR + ApiResourceNameEnum.ALL_METADATA;
  }

  static get publicMetadata(): string {
    return BaseResourceApiEnum.access + SEPARATOR + ApiResourceNameEnum.PUBLIC_METADATA;
  }

  static get privateMetadata(): string {
    return BaseResourceApiEnum.access + SEPARATOR + ApiResourceNameEnum.PRIVATE_METADATA;
  }

  static get publicMetadataSearch(): string {
    return AccessResourceApiEnum.publicMetadata + SEPARATOR + search;
  }

  static get allMetadataSearch(): string {
    return AccessResourceApiEnum.allMetadata + SEPARATOR + search;
  }

  static get privateMetadataSearch(): string {
    return AccessResourceApiEnum.privateMetadata + SEPARATOR + search;
  }

  static get oaiSets(): string {
    return BaseResourceApiEnum.access + SEPARATOR + ApiResourceNameEnum.OAI_SETS;
  }

  static get orders(): string {
    return BaseResourceApiEnum.access + SEPARATOR + ApiResourceNameEnum.ORDER;
  }

  static get organizationalUnits(): string {
    return BaseResourceApiEnum.access + SEPARATOR + ApiResourceNameEnum.ORG_UNIT;
  }
}

export class DataManagementApiEnum implements ResourceApiEnum {
  static get root(): string {
    return BaseResourceApiEnum.dataMgmt;
  }

  static get publicDiSearch(): string {
    return BaseResourceApiEnum.dataMgmt + SEPARATOR + ApiResourceNameEnum.DI_PUBLIC + SEPARATOR + search;
  }
}

export class AdminResourceApiEnum implements ResourceApiEnum {
  // Resource
  static get root(): string {
    return BaseResourceApiEnum.admin;
  }

  static get fundingAgencies(): string {
    return BaseResourceApiEnum.admin + SEPARATOR + ApiResourceNameEnum.FUNDING_AGENCY;
  }

  static get institutions(): string {
    return BaseResourceApiEnum.admin + SEPARATOR + ApiResourceNameEnum.INSTITUTION;
  }

  static get languages(): string {
    return BaseResourceApiEnum.admin + SEPARATOR + ApiResourceNameEnum.LANGUAGE;
  }

  static get licenses(): string {
    return BaseResourceApiEnum.admin + SEPARATOR + ApiResourceNameEnum.LICENSE;
  }

  static get oauth2Clients(): string {
    return BaseResourceApiEnum.admin + SEPARATOR + ApiResourceNameEnum.OAUTH2_CLIENT;
  }

  static get organizationalUnits(): string {
    return BaseResourceApiEnum.admin + SEPARATOR + ApiResourceNameEnum.ORG_UNIT;
  }

  static get authorizedOrganizationalUnits(): string {
    return BaseResourceApiEnum.admin + SEPARATOR + ApiResourceNameEnum.AUTHORIZED_ORG_UNIT;
  }

  static get people(): string {
    return BaseResourceApiEnum.admin + SEPARATOR + ApiResourceNameEnum.PERSON;
  }

  static get preservationPolicies(): string {
    return BaseResourceApiEnum.admin + SEPARATOR + ApiResourceNameEnum.PRES_POLICY;
  }

  static get roles(): string {
    return BaseResourceApiEnum.admin + SEPARATOR + ApiResourceNameEnum.ROLE;
  }

  static get submissionAgreements(): string {
    return BaseResourceApiEnum.admin + SEPARATOR + ApiResourceNameEnum.SUBMISSION_AGREEMENT;
  }

  static get submissionPolicies(): string {
    return BaseResourceApiEnum.admin + SEPARATOR + ApiResourceNameEnum.SUB_POLICY;
  }

  static get users(): string {
    return BaseResourceApiEnum.admin + SEPARATOR + ApiResourceNameEnum.USER;
  }
}

export class AdminSubResourceApiEnum implements SubResourceApiEnum {
  // Sub Resource
  static get organizationalUnitsSubResource(): string {
    return AdminResourceApiEnum.organizationalUnits + parentIdInUrl + subResource;
  }

  static get organizationalUnitsSubmissionPolicy(): string {
    return AdminResourceApiEnum.organizationalUnits + parentIdInUrl + ApiResourceNameEnum.SUB_POLICY;
  }

  static get organizationalUnitsSubmissionAgreement(): string {
    return AdminResourceApiEnum.organizationalUnits + parentIdInUrl + ApiResourceNameEnum.SUBMISSION_AGREEMENT;
  }
}

export class ArchivalStorageResourceApiEnum implements ResourceApiEnum {
  static get root(): string {
    return BaseResourceApiEnum.archivalStorage;
  }

  static get aip(): string {
    return BaseResourceApiEnum.archivalStorage + SEPARATOR + ApiResourceNameEnum.AIP;
  }

  static get storedAip(): string {
    return BaseResourceApiEnum.archivalStorage + SEPARATOR + ApiResourceNameEnum.STORED_AIP;
  }
}

export class IngestResourceApiEnum implements ResourceApiEnum {
  static get root(): string {
    return BaseResourceApiEnum.ingest;
  }

  static get sip(): string {
    return BaseResourceApiEnum.ingest + SEPARATOR + ApiResourceNameEnum.SIP;
  }
}

export class PreIngestResourceApiEnum implements ResourceApiEnum {
  static get root(): string {
    return BaseResourceApiEnum.preingest;
  }

  static get deposits(): string {
    return BaseResourceApiEnum.preingest + SEPARATOR + ApiResourceNameEnum.DEPOSIT;
  }

  static get depositDataFile(): string {
    return BaseResourceApiEnum.preingest + SEPARATOR + ApiResourceNameEnum.DEPOSIT + parentIdInUrl + ApiResourceNameEnum.DATAFILE;

  }
}

export class PreservationPlanningResourceApiEnum implements ResourceApiEnum {
  static get root(): string {
    return BaseResourceApiEnum.preservationPlanning;
  }

  static get preservationJobs(): string {
    return BaseResourceApiEnum.preservationPlanning + SEPARATOR + ApiResourceNameEnum.PRES_JOB;
  }
}

export class ResourceSrvResourceApiEnum implements ResourceApiEnum {
  static get root(): string {
    return BaseResourceApiEnum.resourceSrv;
  }

  static get applicationRoles(): string {
    return BaseResourceApiEnum.resourceSrv + SEPARATOR + ApiResourceNameEnum.ROLE_APPLICATION;
  }

  static get basicAuthClients(): string {
    return BaseResourceApiEnum.resourceSrv + SEPARATOR + ApiResourceNameEnum.BASIC_AUTH_CLIENT;
  }
}
