import {Injectable} from "@angular/core";
import {
  ActivatedRouteSnapshot,
  CanActivate,
  Router,
} from "@angular/router";
import {ApplicationRoleEnum} from "@app/shared/enums/application-role.enum";
import {DlcmData} from "@app/shared/models/dlcm-route.model";
import {LocalStateModel} from "@app/shared/models/local-state.model";
import {PermissionUtil} from "@app/shared/utils/permission.util";
import {Store} from "@ngxs/store";
import {isNullOrUndefined} from "util";

@Injectable({
  providedIn: "root",
})
export class ApplicationRoleGuardService implements CanActivate {
  constructor(public router: Router, public store: Store) {
  }

  canActivate(route: ActivatedRouteSnapshot): boolean {
    const data = route.data as DlcmData;
    const permission = isNullOrUndefined(data) ? undefined : data.permission;
    const currentUserRole = this.store.selectSnapshot<ApplicationRoleEnum[]>((state: LocalStateModel) => state.application.userRoles);
    return PermissionUtil.isUserHavePermission(permission, currentUserRole);
  }
}
