import {Injectable} from "@angular/core";
import {
  CanActivate,
  Router,
} from "@angular/router";
import {RoutesEnum} from "@app/shared/enums/routes.enum";
import {LocalStateModel} from "@app/shared/models/local-state.model";
import {Navigate} from "@ngxs/router-plugin";
import {Store} from "@ngxs/store";

@Injectable({
  providedIn: "root",
})
export class AuthGuardService implements CanActivate {
  constructor(public router: Router, public store: Store) {
  }

  canActivate(): boolean {
    const isLoggedIn: boolean = this.store.selectSnapshot<boolean>((state: LocalStateModel) => state.application.isLoggedIn);
    if (isLoggedIn === false) {
      this.store.dispatch(new Navigate([RoutesEnum.index]));
    }
    return isLoggedIn;
  }

}
