import {Injectable} from "@angular/core";
import {
  ActivatedRouteSnapshot,
  CanActivate,
  Router,
} from "@angular/router";
import {AppState} from "@app/stores/app.state";
import {DepositAction} from "@deposit/stores/deposit.action";
import {DepositState} from "@deposit/stores/deposit.state";
import {Store} from "@ngxs/store";
import {AppRoutesEnum} from "@shared/enums/routes.enum";
import {Observable} from "rxjs";
import {map} from "rxjs/operators";
import {ApiService} from "solidify-frontend";

@Injectable({
  providedIn: "root",
})
export class DepositRoleGuardService implements CanActivate {
  constructor(public router: Router,
              public store: Store,
              public apiService: ApiService) {
  }

  canActivate(route: ActivatedRouteSnapshot): Observable<boolean> {
    const listAuthorizedOrgUnitId = this.store.selectSnapshot(AppState.listAuthorizedOrganizationalUnitId);
    const depositId: string = route.params[AppRoutesEnum.paramIdWithoutPrefixParam];
    return this.store.dispatch(new DepositAction.GetById(depositId)).pipe(
      map(state => {
        const currentDeposit = this.store.selectSnapshot(DepositState.current);
        return listAuthorizedOrgUnitId.includes(currentDeposit.organizationalUnitId);
      }),
    );
  }
}
