import {HttpErrorResponse} from "@angular/common/http";
import {
  Injectable,
  Injector,
} from "@angular/core";
import {AppAction} from "@app/stores/app.action";
import {Store} from "@ngxs/store";
import {
  ENVIRONMENT,
  ErrorDto,
  ErrorHelper,
  GlobalErrorsHandlerService,
  HttpStatus,
} from "solidify-frontend";

@Injectable({
  providedIn: "root",
})
export class DlcmErrorsHandlerService extends GlobalErrorsHandlerService {
  // Error handling is important and needs to be loaded first.
  // Because of this we should manually inject the services with Injector.
  constructor(protected injector: Injector) {
    super(injector);
  }

  handleError(error: Error | HttpErrorResponse): void {
    const environment = this.injector.get(ENVIRONMENT);
    if (error === environment.errorToSkipInErrorHandler) {
      return;
    }
    const httpErrorKeyToSkipInErrorHandler = environment.httpErrorKeyToSkipInErrorHandler;

    if (error instanceof HttpErrorResponse) {
      const errorDto = error.error as ErrorDto;
      if (ErrorHelper.isErrorToTreat(error, httpErrorKeyToSkipInErrorHandler)) {
        // Server Error
        if (error.status === HttpStatus.UNAUTHORIZED) {
          if (error.error.error === ErrorHelper.INVALID_TOKEN_ERROR) {
            const store = this.injector.get(Store);
            store.dispatch(new AppAction.Logout());
          }
        }
      }
    }

    super.handleError(error);
    console.error(error);
  }
}
