import {
  FundingAgency,
  Institution,
} from "@app/generated-api";
import {AdminResourceApiEnum} from "@app/shared/enums/api.enum";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {RoutesEnum} from "@app/shared/enums/routes.enum";
import {
  Actions,
  State,
  Store,
} from "@ngxs/store";
import {sharedFundingAgencyActionNameSpace} from "@shared/stores/funding-agency/shared-funding-agency.action";
import {
  ApiService,
  defaultResourceStateInitValue,
  NotificationService,
  QueryParameters,
  ResourceState,
  ResourceStateModel,
  TRANSLATE,
} from "solidify-frontend";
import {environment} from "../../../../environments/environment";

export interface SharedFundingAgencyStateModel extends ResourceStateModel<FundingAgency> {
}

@State<SharedFundingAgencyStateModel>({
  name: LocalStateEnum.shared_fundingAgency,
  defaults: {
    ...defaultResourceStateInitValue,
    queryParameters: new QueryParameters(environment.defaultEnumValuePageSizeOption),
  },
})
export class SharedFundingAgencyState extends ResourceState<FundingAgency> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: sharedFundingAgencyActionNameSpace,
      routeRedirectUrlAfterSuccessAction: RoutesEnum.homePage,
      notificationResourceCreateTextToTranslate: TRANSLATE("shared.fundingAgency.notification.resource.create"),
      notificationResourceDeleteTextToTranslate: TRANSLATE("shared.fundingAgency.notification.resource.delete"),
      notificationResourceUpdateTextToTranslate: TRANSLATE("shared.fundingAgency.notification.resource.update"),
    });
  }

  protected get _urlResource(): string {
    return AdminResourceApiEnum.fundingAgencies;
  }
}
