import {Institution} from "@app/generated-api";
import {AdminResourceApiEnum} from "@app/shared/enums/api.enum";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {RoutesEnum} from "@app/shared/enums/routes.enum";
import {sharedInstitutionActionNameSpace} from "@app/shared/stores/institution/shared-institution.action";
import {
  Actions,
  State,
  Store,
} from "@ngxs/store";
import {
  ApiService,
  defaultResourceStateInitValue,
  NotificationService,
  QueryParameters,
  ResourceState,
  ResourceStateModel,
  TRANSLATE,
} from "solidify-frontend";
import {environment} from "../../../../environments/environment";

export interface SharedInstitutionStateModel extends ResourceStateModel<Institution> {
}

@State<SharedInstitutionStateModel>({
  name: LocalStateEnum.shared_institution,
  defaults: {
    ...defaultResourceStateInitValue,
    queryParameters: new QueryParameters(environment.defaultEnumValuePageSizeOption),
  },
})
export class SharedInstitutionState extends ResourceState<Institution> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: sharedInstitutionActionNameSpace,
      routeRedirectUrlAfterSuccessAction: RoutesEnum.homePage,
      notificationResourceCreateTextToTranslate: TRANSLATE("shared.institution.notification.resource.create"),
      notificationResourceDeleteTextToTranslate: TRANSLATE("shared.institution.notification.resource.delete"),
      notificationResourceUpdateTextToTranslate: TRANSLATE("shared.institution.notification.resource.update"),
    });
  }

  protected get _urlResource(): string {
    return AdminResourceApiEnum.institutions;
  }
}
