import {License} from "@app/generated-api";
import {AdminResourceApiEnum} from "@app/shared/enums/api.enum";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {RoutesEnum} from "@app/shared/enums/routes.enum";
import {sharedLicenseActionNameSpace} from "@app/shared/stores/license/shared-license.action";
import {
  Actions,
  Selector,
  State,
  Store,
} from "@ngxs/store";
import {
  ApiService,
  defaultResourceStateInitValue,
  NotificationService,
  QueryParameters,
  ResourceState,
  ResourceStateModel,
  StoreUtil,
  TRANSLATE,
} from "solidify-frontend";
import {environment} from "../../../../environments/environment";

export interface SharedLicenseStateModel extends ResourceStateModel<License> {
}

@State<SharedLicenseStateModel>({
  name: LocalStateEnum.shared_license,
  defaults: {
    ...defaultResourceStateInitValue,
    queryParameters: new QueryParameters(environment.defaultEnumValuePageSizeOption),
  },
})
export class SharedLicenseState extends ResourceState<License> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: sharedLicenseActionNameSpace,
      routeRedirectUrlAfterSuccessAction: RoutesEnum.homePage,
      notificationResourceCreateTextToTranslate: TRANSLATE("shared.license.notification.resource.create"),
      notificationResourceDeleteTextToTranslate: TRANSLATE("shared.license.notification.resource.delete"),
      notificationResourceUpdateTextToTranslate: TRANSLATE("shared.license.notification.resource.update"),
    });
  }

  protected get _urlResource(): string {
    return AdminResourceApiEnum.licenses;
  }

  @Selector()
  static list<T>(state: ResourceStateModel<T>): T[] {
    return state.list;
  }

  @Selector()
  static total<T>(state: ResourceStateModel<T>): number {
    return state.total;
  }

  @Selector()
  static current<T>(state: ResourceStateModel<T>): T {
    return state.current;
  }

  @Selector()
  static isLoading<T>(state: ResourceStateModel<T>): boolean {
    return StoreUtil.isLoadingState(state);
  }
}
