import {PersonExtended} from "@admin/models/person-extended.model";
import {
  Institution,
  Person,
} from "@app/generated-api";
import {
  BaseAction,
  BaseSubAction,
  ResourceAction,
  ResourceNameSpace,
  TypeDefaultAction,
} from "solidify-frontend";
import {LocalStateEnum} from "../../enums/local-state.enum";

const state = LocalStateEnum.shared_person;

export namespace SharedPersonAction {
  @TypeDefaultAction(state)
  export class LoadResource extends ResourceAction.LoadResource {
  }

  @TypeDefaultAction(state)
  export class LoadResourceSuccess extends ResourceAction.LoadResourceSuccess {
  }

  @TypeDefaultAction(state)
  export class LoadResourceFail extends ResourceAction.LoadResourceFail {
  }

  @TypeDefaultAction(state)
  export class ChangeQueryParameters extends ResourceAction.ChangeQueryParameters {
  }

  @TypeDefaultAction(state)
  export class GetAll extends ResourceAction.GetAll {
  }

  @TypeDefaultAction(state)
  export class GetAllSuccess extends ResourceAction.GetAllSuccess<PersonExtended> {
  }

  @TypeDefaultAction(state)
  export class GetAllFail extends ResourceAction.GetAllFail<PersonExtended> {
  }

  @TypeDefaultAction(state)
  export class GetById extends ResourceAction.GetById {
  }

  @TypeDefaultAction(state)
  export class GetByIdSuccess extends ResourceAction.GetByIdSuccess<PersonExtended> {
  }

  @TypeDefaultAction(state)
  export class GetByIdFail extends ResourceAction.GetByIdFail<PersonExtended> {
  }

  @TypeDefaultAction(state)
  export class Create extends ResourceAction.Create<PersonExtended> {
  }

  @TypeDefaultAction(state)
  export class CreateSuccess extends ResourceAction.CreateSuccess<PersonExtended> {
  }

  @TypeDefaultAction(state)
  export class CreateFail extends ResourceAction.CreateFail<PersonExtended> {
  }

  @TypeDefaultAction(state)
  export class Update extends ResourceAction.Update<PersonExtended> {
  }

  @TypeDefaultAction(state)
  export class UpdateSuccess extends ResourceAction.UpdateSuccess<PersonExtended> {
  }

  @TypeDefaultAction(state)
  export class UpdateFail extends ResourceAction.UpdateFail<PersonExtended> {
  }

  @TypeDefaultAction(state)
  export class Delete extends ResourceAction.Delete {
  }

  @TypeDefaultAction(state)
  export class DeleteSuccess extends ResourceAction.DeleteSuccess {
  }

  @TypeDefaultAction(state)
  export class DeleteFail extends ResourceAction.DeleteFail {
  }

  @TypeDefaultAction(state)
  export class Clean extends ResourceAction.Clean {
  }

  export class Search extends BaseAction {
    static readonly type: string = `[${state}] Search`;

    constructor(public person: PersonExtended) {
      super();
    }
  }

  export class SearchSuccess extends BaseSubAction<Search> {
    static readonly type: string = `[${state}] Search Success`;
  }

  export class SearchFail extends BaseSubAction<Search> {
    static readonly type: string = `[${state}] Search Fail`;
  }

  export class SearchPerson extends BaseAction {
    static readonly type: string = `[${state}] Search Person`;

    constructor(public person: PersonExtended) {
      super();
    }
  }

  export class SearchPersonSuccess extends BaseSubAction<SearchPerson> {
    static readonly type: string = `[${state}] Search Person Success`;

    constructor(public parentAction: SearchPerson, public list: PersonExtended[]) {
      super(parentAction);
    }
  }

  export class SearchPersonFail extends BaseSubAction<SearchPerson> {
    static readonly type: string = `[${state}] Search Person Fail`;
  }

  export class SearchInstitutions extends BaseAction {
    static readonly type: string = `[${state}] Search Institutions`;
  }

  export class SearchInstitutionsSuccess extends BaseSubAction<SearchInstitutions> {
    static readonly type: string = `[${state}] Search Institutions Success`;
  }

  export class SearchInstitutionsFail extends BaseSubAction<SearchInstitutions> {
    static readonly type: string = `[${state}] Search Institutions Fail`;
  }

  export class SearchPersonInstitution extends BaseAction {
    static readonly type: string = `[${state}] Search Person Institution`;

    constructor(public person: PersonExtended) {
      super();
    }
  }

  export class SearchPersonInstitutionSuccess extends BaseSubAction<SearchPersonInstitution> {
    static readonly type: string = `[${state}] Search Person Institution Success`;

    constructor(public parentAction: SearchPersonInstitution, public person: PersonExtended, public institutions: Institution[]) {
      super(parentAction);
    }
  }

  export class SearchPersonInstitutionFail extends BaseSubAction<SearchPersonInstitution>  {
    static readonly type: string = `[${state}] Search Person Institution Fail`;
  }
}

export const sharedPersonActionNameSpace: ResourceNameSpace = SharedPersonAction;
