import {PreservationPolicy} from "@app/generated-api";
import {AdminResourceApiEnum} from "@app/shared/enums/api.enum";
import {RoutesEnum} from "@app/shared/enums/routes.enum";
import {sharedPreservationPolicyActionNameSpace} from "@app/shared/stores/preservation-policy/shared-preservation-policy.action";
import {
  Actions,
  State,
  Store,
} from "@ngxs/store";
import {
  ApiService,
  defaultResourceStateInitValue,
  NotificationService,
  QueryParameters,
  ResourceState,
  ResourceStateModel,
  TRANSLATE,
} from "solidify-frontend";
import {environment} from "../../../../environments/environment";
import {LocalStateEnum} from "../../enums/local-state.enum";

export interface SharedPreservationPolicyStateModel extends ResourceStateModel<PreservationPolicy> {
}

@State<SharedPreservationPolicyStateModel>({
  name: LocalStateEnum.shared_preservationPolicy,
  defaults: {
    ...defaultResourceStateInitValue,
    queryParameters: new QueryParameters(environment.defaultEnumValuePageSizeOption),
  },
})
export class SharedPreservationPolicyState extends ResourceState<PreservationPolicy> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: sharedPreservationPolicyActionNameSpace,
      routeRedirectUrlAfterSuccessAction: RoutesEnum.homePage,
      notificationResourceCreateTextToTranslate: TRANSLATE("shared.preservationPolicy.notification.resource.create"),
      notificationResourceDeleteTextToTranslate: TRANSLATE("shared.preservationPolicy.notification.resource.delete"),
      notificationResourceUpdateTextToTranslate: TRANSLATE("shared.preservationPolicy.notification.resource.update"),
    });
  }

  protected get _urlResource(): string {
    return AdminResourceApiEnum.preservationPolicies;
  }
}
