import {SubmissionPolicy} from "@app/generated-api";
import {AdminResourceApiEnum} from "@app/shared/enums/api.enum";
import {RoutesEnum} from "@app/shared/enums/routes.enum";
import {sharedSubmissionPolicyActionNameSpace} from "@app/shared/stores/submission-policy/shared-submission-policy.action";
import {
  Actions,
  State,
  Store,
} from "@ngxs/store";
import {
  ApiService,
  defaultResourceStateInitValue,
  NotificationService,
  QueryParameters,
  ResourceState,
  ResourceStateModel,
  TRANSLATE,
} from "solidify-frontend";
import {environment} from "../../../../environments/environment";
import {LocalStateEnum} from "../../enums/local-state.enum";

export interface SharedSubmissionPolicyStateModel extends ResourceStateModel<SubmissionPolicy> {
}

@State<SharedSubmissionPolicyStateModel>({
  name: LocalStateEnum.shared_submissionPolicy,
  defaults: {
    ...defaultResourceStateInitValue,
    queryParameters: new QueryParameters(environment.defaultEnumValuePageSizeOption),
  },
})
export class SharedSubmissionPolicyState extends ResourceState<SubmissionPolicy> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: sharedSubmissionPolicyActionNameSpace,
      routeRedirectUrlAfterSuccessAction: RoutesEnum.homePage,
      notificationResourceCreateTextToTranslate: TRANSLATE("shared.submissionPolicy.notification.resource.create"),
      notificationResourceDeleteTextToTranslate: TRANSLATE("shared.submissionPolicy.notification.resource.delete"),
      notificationResourceUpdateTextToTranslate: TRANSLATE("shared.submissionPolicy.notification.resource.update"),
    });
  }

  protected get _urlResource(): string {
    return AdminResourceApiEnum.submissionPolicies;
  }
}
