import {OrganizationalUnit} from "@app/generated-api";
import {AdminResourceApiEnum} from "@app/shared/enums/api.enum";
import {appAuthorizedOrganizationalUnit} from "@app/stores/authorized-organizational-unit/app-authorized-organizational-unit.action";
import {
  Actions,
  State,
  Store,
} from "@ngxs/store";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {
  ApiService,
  defaultResourceStateInitValue,
  NotificationService,
  QueryParameters,
  ResourceState,
  ResourceStateModel,
} from "solidify-frontend";
import {environment} from "../../../environments/environment";

export interface AppAuthorizedOrganizationalUnitStateModel extends ResourceStateModel<OrganizationalUnit> {
}

@State<AppAuthorizedOrganizationalUnitStateModel>({
  name: LocalStateEnum.application_authorizedOrganizationalUnit,
  defaults: {
    ...defaultResourceStateInitValue,
    queryParameters: new QueryParameters(environment.defaultEnumValuePageSizeOption),
  },
})
export class AppAuthorizedOrganizationalUnitState extends ResourceState<OrganizationalUnit> {

  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: appAuthorizedOrganizationalUnit,
      routeRedirectUrlAfterSuccessAction: undefined,
      notificationResourceCreateTextToTranslate: undefined,
      notificationResourceDeleteTextToTranslate: undefined,
      notificationResourceUpdateTextToTranslate: undefined,
    });
  }

  protected get _urlResource(): string {
    return AdminResourceApiEnum.authorizedOrganizationalUnits;
  }
}
