import {Person} from "@app/generated-api";
import {LocalStateEnum} from "@shared/enums/local-state.enum";
import {UserExtended} from "@shared/models/business/user-extended.model";
import {
  BaseAction,
  BaseSubAction,
  ResourceAction,
  ResourceNameSpace,
  TypeDefaultAction,
} from "solidify-frontend";

const state = LocalStateEnum.application_user;

export namespace AppUserAction {
  @TypeDefaultAction(state)
  export class LoadResource extends ResourceAction.LoadResource {
  }

  @TypeDefaultAction(state)
  export class LoadResourceSuccess extends ResourceAction.LoadResourceSuccess {
  }

  @TypeDefaultAction(state)
  export class LoadResourceFail extends ResourceAction.LoadResourceFail {
  }

  @TypeDefaultAction(state)
  export class ChangeQueryParameters extends ResourceAction.ChangeQueryParameters {
  }

  @TypeDefaultAction(state)
  export class GetAll extends ResourceAction.GetAll {
  }

  @TypeDefaultAction(state)
  export class GetAllSuccess extends ResourceAction.GetAllSuccess<Person> {
  }

  @TypeDefaultAction(state)
  export class GetAllFail extends ResourceAction.GetAllFail<Person> {
  }

  @TypeDefaultAction(state)
  export class GetById extends ResourceAction.GetById {
  }

  @TypeDefaultAction(state)
  export class GetByIdSuccess extends ResourceAction.GetByIdSuccess<Person> {
  }

  @TypeDefaultAction(state)
  export class GetByIdFail extends ResourceAction.GetByIdFail<Person> {
  }

  @TypeDefaultAction(state)
  export class Create extends ResourceAction.Create<Person> {
  }

  @TypeDefaultAction(state)
  export class CreateSuccess extends ResourceAction.CreateSuccess<Person> {
  }

  @TypeDefaultAction(state)
  export class CreateFail extends ResourceAction.CreateFail<Person> {
  }

  @TypeDefaultAction(state)
  export class Update extends ResourceAction.Update<Person> {
  }

  @TypeDefaultAction(state)
  export class UpdateSuccess extends ResourceAction.UpdateSuccess<Person> {
  }

  @TypeDefaultAction(state)
  export class UpdateFail extends ResourceAction.UpdateFail<Person> {
  }

  @TypeDefaultAction(state)
  export class Delete extends ResourceAction.Delete {
  }

  @TypeDefaultAction(state)
  export class DeleteSuccess extends ResourceAction.DeleteSuccess {
  }

  @TypeDefaultAction(state)
  export class DeleteFail extends ResourceAction.DeleteFail {
  }

  @TypeDefaultAction(state)
  export class Clean extends ResourceAction.Clean {
  }

  export class GetCurrentUser extends BaseAction {
    static readonly type: string = `[${state}] Get Current User`;
  }

  export class GetCurrentUserSuccess extends BaseSubAction<GetCurrentUser> {
    static readonly type: string = `[${state}] Get Current User Success`;

    constructor(public parentAction: GetCurrentUser, public user: UserExtended) {
      super(parentAction);
    }
  }

  export class GetCurrentUserFail extends BaseSubAction<GetCurrentUser> {
    static readonly type: string = `[${state}] Get Current User Fail`;
  }
}

export const appUserActionNameSpace: ResourceNameSpace = AppUserAction;
