import {AdminResourceApiEnum} from "@app/shared/enums/api.enum";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {RoutesEnum} from "@app/shared/enums/routes.enum";
import {AppPersonAction} from "@app/stores/person/app-person.action";
import {
  AppUserAction,
  appUserActionNameSpace,
} from "@app/stores/user/app-user.action";
import {
  Action,
  Actions,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {ApiActionEnum} from "@shared/enums/api-action.enum";
import {UserExtended} from "@shared/models/business/user-extended.model";
import {Observable} from "rxjs";
import {
  catchError,
  tap,
} from "rxjs/operators";
import {
  ApiService,
  CollectionTyped,
  defaultResourceStateInitValue,
  NotificationService,
  QueryParameters,
  ResourceState,
  ResourceStateModel,
  TRANSLATE,
  urlSeparator,
} from "solidify-frontend";
import {environment} from "../../../environments/environment";

export interface AppUserStateModel extends ResourceStateModel<UserExtended> {
}

@State<AppUserStateModel>({
  name: LocalStateEnum.application_user,
  defaults: {
    ...defaultResourceStateInitValue,
    queryParameters: new QueryParameters(environment.defaultEnumValuePageSizeOption),
  },
})
export class AppUserState extends ResourceState<UserExtended> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: appUserActionNameSpace,
      routeRedirectUrlAfterSuccessAction: RoutesEnum.homePage,
      notificationResourceCreateTextToTranslate: TRANSLATE("app.user.notification.resource.create"),
      notificationResourceDeleteTextToTranslate: TRANSLATE("app.user.notification.resource.delete"),
      notificationResourceUpdateTextToTranslate: TRANSLATE("app.user.notification.resource.update"),
    });
  }

  protected get _urlResource(): string {
    return AdminResourceApiEnum.users;
  }

  @Action(AppUserAction.GetCurrentUser)
  getCurrentUser(ctx: StateContext<AppUserStateModel>, action: AppUserAction.GetCurrentUser): Observable<CollectionTyped<UserExtended>> {
    return this.apiService.get<UserExtended>(AdminResourceApiEnum.users + urlSeparator + ApiActionEnum.AUTHENTICATED, null)
      .pipe(
        tap((user: UserExtended) => {
          ctx.dispatch(new AppUserAction.GetCurrentUserSuccess(action, user));
        }),
        catchError(error => {
          ctx.dispatch(new AppUserAction.GetCurrentUserFail(action));
          throw error;
        }),
      );
  }

  @Action(AppUserAction.GetCurrentUserSuccess)
  getCurrentUserSuccess(ctx: StateContext<AppUserStateModel>, action: AppUserAction.GetCurrentUserSuccess): void {
    ctx.patchState({
      current: action.user,
    });

    ctx.dispatch(new AppPersonAction.AddCurrentPerson(action.user.person));
  }
}
